import {signaling, randArrayBuffer} from '../../config'
import {WebChannel} from '../../../src/WebChannel'


const MULTIPLIER_FOR_128KB_STRING = 64

describe('Large data -> ', () => {
  let wc1, wc2

  beforeAll((done) => {
    // Peer #1
    wc1 = new WebChannel({signaling})
    wc1.openForJoining().then((data) => {
      // Peer #2
      wc2 = new WebChannel({signaling})
      wc2.join(data.key).then(() => {
        done()
      })
        .catch(done.fail)
    }).catch(done.fail)
  })

  describe('Should send & receive String between 2 peers -> ', () => {
    it('~17kb', (done) => {
      wc1.onMessage = (id, msg) => {
        expect(msg).toEqual(smallStr)
        done()
      }
      wc2.send(smallStr)
    })

    it('~8192kb', (done) => {
      let fullStr = ''
      let startTime
      for (let i = 0; i < MULTIPLIER_FOR_128KB_STRING; i++) {
        fullStr += bigStr
      }
      wc1.onMessage = (id, msg) => {
        console.info('~' + (128 * MULTIPLIER_FOR_128KB_STRING) + 'kb String sent between 2 peers in: ' + (Date.now() - startTime) + ' ms')
        expect(msg).toEqual(fullStr)
        done()
      }
      startTime = Date.now()
      wc2.send(fullStr)
    })
  })

  function isEqual (buffer1, buffer2) {
    let array1 = new Uint8Array(buffer1)
    let array2 = new Uint8Array(buffer2)

    return array1.every((value, index) => {
      return value === array2[index]
    })
  }

  describe('Should send & receive ArrayBuffer between 2 peers -> ', () => {
    let smallBuffer = randArrayBuffer(10000, 10000)
    let bigBuffer = randArrayBuffer(8388608, 8400000)

    it('between 16kb & 20kb', (done) => {
      wc1.onMessage = (id, msg) => {
        expect(isEqual(msg, smallBuffer)).toBeTruthy()
        done()
      }
      wc2.send(smallBuffer)
    }, 10000)

    it('~8192kb', (done) => {
      let startTime
      wc1.onMessage = (id, msg) => {
        console.info(bigBuffer.byteLength + ' bytes sent between 2 peers in: ' + (Date.now() - startTime) + ' ms')
        expect(isEqual(msg, bigBuffer)).toBeTruthy()
        done()
      }
      startTime = Date.now()
      wc2.send(bigBuffer)
    }, 10000)
  })
})

const smallStr =
`Clint Eastwood (prononcé en anglais : /klɪnt istwʊd/), né le 31 mai 1930 à San Francisco (Californie), est un acteur, réalisateur, compositeur et producteur de cinéma américain.
Autodidacte, il entre grâce à des amis au studio Universal où il interprète d’abord de petits rôles dans des séries B, puis l’un des rôles phares d'une longue série, Rawhide. Il se fait alors remarquer par Sergio Leone qui l’embauche pour la Trilogie du dollar (Pour une poignée de dollars, Et pour quelques dollars de plus et Le Bon, la Brute et le Truand). Devenu célèbre, il interprète de nombreux rôles, d’abord pour Universal, puis pour Warner Bros., notamment ceux de L'Inspecteur Harry. En 1968, il devient producteur avec la création de la société Malpaso et réalise son premier film en 1971, avec Un frisson dans la nuit. Aujourd'hui, avec plus de quatre-vingt films à son actif, parmi lesquels Impitoyable, Sur la route de Madison ou encore Mystic River et plus récemment Million Dollar Baby, Gran Torino, Invictus et American Sniper, Clint Eastwood figure parmi les cinéastes les plus reconnus au monde.
D'abord connu pour ses rôles d'antihéros volontiers redresseur de torts et tragiques, dans des films d'action violents ou des westerns tels que L'Homme des Hautes Plaines ou encore Pale Rider, il a ensuite endossé des rôles plus touchants dans des films empreints d'un certain classicisme, influencés par le cinéma de John Ford et de Howard Hawks. Il est également connu pour ses comédies telles que Doux, dur et dingue et Ça va cogner. Il a ainsi été récompensé à de nombreuses reprises, remportant notamment quatre Oscars, cinq Golden Globes, trois Césars et la Palme d'honneur au Festival de Cannes en 2009.
Étymologie du nom
Selon Le Robert des noms propres, le nom Eastwood, vient du vieil anglais qui signifie « bois (wudu) de l'Est (ēast) » ou du vieux scandinave °veit « prairie ».
Biographie
Origines

Hôpital Saint Francis où naît Clint Eastwood
Clint Eastwood a toujours été mystérieux sur ses origines, sa vie privée et son passé. Eastwood est sélectif car il veut être celui qui sait sans divulguer. Lors d'interviews, il dévoile seulement la partie de son arbre généalogique qui met en valeur son image,. Pourtant, les origines d'Eastwood suivent de près l'histoire américaine. Ses ancêtres arrivent en Amérique du Nord au milieu du xviie siècle. Ils font partie des premiers colons à se lancer dans la conquête de l'Ouest. Sa famille se partage donc entre des membres installés à New York, dans l'Ohio, dans le Michigan, en Virginie, dans l'Illinois, en Louisiane, au Kansas, dans le Colorado, le Nevada, en Californie et enfin en Alaska. Bien avant que Clint Eastwood naisse, sa famille est marquée par le monde du spectacle. Le premier Eastwood né en Amérique est Lewis Eastwood. Ses parents sont venus d'Angleterre ; ils sont toutefois d'origine irlandaise,. Bien qu'il ait déclaré à la presse être « le premier de la famille à avoir réussi », Clint Eastwood est bien loin de la vérité : à la fin du xviiie siècle, Lewis Eastwood est devenu un entrepreneur renommé, classé cent troisième parmi les mille deux cents commissionnaires de la ville de New York. L'un des petits-fils de Lewis, Asa Bedesco Eastwood, l'arrière-grand-père de Clint, quitte la ville pour devenir mineur. C'est à cause de lui que le réalisateur a souvent montré, à travers ses films, une tendresse particulière à l'égard des mineurs, comme dans La Kermesse de l'Ouest (1969), L'Homme des Hautes Plaines (1973) ou encore dans Pale Rider, le cavalier solitaire (1985). Habitué du commerce, Burr, l'un des fils d'Asa Bedesco, quitte sa famille pour travailler comme magasinier, emploi dans lequel il monte rapidement l'échelle sociale. Il épouse Jessie Anderson, une immigrante d'Écosse, qui lui donne deux fils, dont Clinton. Clinton se marie en 1927 avec Margaret Ruth Runner, une femme de la haute société. Ils donnent naissance à un garçon qui leur dédie plus tard l'Oscar du meilleur film qu'il remporte pour Impitoyable,, :
« Cette victoire est simplement merveilleuse, je la décerne à toutes les personnes auxquelles je pourrais penser. […] Durant cette année de la femme, la plus grande femme sur la planète est ici ce soir — ma mère, Ruth. »
Clinton Eastwood Jr. est donc né le 31 mai 1930, à l'hôpital Saint Francis de San Francisco. À cette époque, le nourrisson était déjà célèbre. Sa mère déclare au journal anglais News of the World : « C'était le plus gros bébé de la maternité. Il pesait 5,2 kg. Les infirmières s'amusaient beaucoup à le montrer aux autres mamans. Elles l'appelaient « Samson ». Il était tellement grand,. » Il fut surnommé Clinton Jr. en hommage à son père, bien que son nom complet soit Clinton Elias Eastwood Jr. Toutefois, il fut surnommé par ses parents « Sonny ». Les liens familiaux sont forts chez les Eastwood, comme l'exprime Ruth lors de la naissance de son fils : « Je suis tombée amoureuse de lui dès qu'il est né ! ». Et cet amour est largement restitué dans tous les films dans lesquels Clint Jr. est impliqué par la suite.
Enfance
La Grande Dépression

Oakland, où Eastwood aurait passé son enfance.
Ses différents attachés de presse ont, durant quarante ans, clamé que Clint Eastwood était originaire d'Oakland, ville ouvrière qui mettrait en valeur la réussite d'Eastwood. Ce dernier a même déclaré dans une interview que s'il traitait si souvent les gens de « trous du cul » dans ses films, c'était probablement à cause de son enfance passée à Oakland. Mais cette information n'est pas vraiment exacte. Dans la biographie écrite par Schickel publiée en 1996, on découvre qu'Eastwood a, en fait, grandi à Piedmont,. Schickel y déclare que les Eastwood ont grandi dans une « modeste maison au toit couvert de bardeaux », mais il précise que « cette maison était [toutefois] située à la limite d'Oakland ». L'enfance de Clint Eastwood est marquée par la Grande Dépression et le passage au cinéma sonore. Les journaux locaux ne traitent guère de la crise. Toutefois le chômage ne cesse d'augmenter. Il atteint un taux de 28 % en Californie. Si Oakland, d'origine ouvrière, est très touchée par la Grande Dépression, Piedmont fait figure de banlieue chic où la crise n'a pas de réel impact sur la vie de tous les jours. Toutefois, les parents du jeune Eastwood quittent la région : Clinton Sr. vient de perdre son emploi de commercial chez East Bay Refrigeration Products,.

Spokane, ville dans laquelle Clinton Sr. trouve un travail temporaire.
Selon les divers témoignages, Clinton Sr. se met en quête de travail partout où il y en a. Il déclare ainsi à son fils : « dans la vie, on n’a rien pour rien », ce que Clint Jr. n’a jamais contesté. C'est d'ailleurs peut-être de ce nomadisme que naît la future passion d'Eastwood pour les westerns. Il n'a ni diplôme universitaire ni qualification professionnelle. Les voyant découragés, le frère de Ruth, la mère de Clint Jr., les dépanne financièrement comme il peut. Il aide par ailleurs Clinton Sr. à trouver un emploi dans une usine de réfrigérateurs à Spokane. Ce dernier enchaîne avec un travail de pompiste sur Sunset Boulevard qu'il obtient grâce à des amis. La famille s'installe alors à Pacific Palisades, un district de Los Angeles. C'est durant cette période que Clint Jr. manque de mourir noyé à l'âge de quatre ans et qu'il assiste à la naissance de sa sœur, Jeanne. L'enfance de Clint Jr. est ainsi marquée par des déménagements incessants dus aux changements de travail de son père : ils vont notamment à Sacramento, et Redding. Ces voyages vont durer près de six ans. Cependant, Schickel déclare dans son livre sur Clint Eastwood qu'« il n'y avait jamais ni panique ni désespoir dans ces déménagements. […] Quand la famille faisait ses paquets, M. Eastwood avait toujours retrouvé un emploi. Et à aucun moment Clint ne s'est senti délaissé ou abandonné durant cette période. ». Au milieu des années 1930, la mère de Clint Eastwood achète la maison de sa tante à Piedmont pour une somme dérisoire. En évoquant cette période, l'acteur déclare au Village Voice, en 1976, que « c'était une époque merdique ». Il ajoute « on n'était pas itinérants. […] C'était pas Les Raisins de la colère, mais c'était pas le luxe non plus » au Rolling Stone.
De retour dans sa ville natale, Clint Eastwood rend souvent visite à sa grand-mère, Virginia May Runner, jusqu'en 1937, date à laquelle cette dernière déménage vers la région rurale, derrière les faubourgs Est d'Oakland. Malgré son départ, Clint Eastwood ne la perd pas de vue pour autant, il va chez elle de temps en temps. C'est durant ces quelques séjours que Clint Eastwood apprend à monter à cheval. Il y apprend également les valeurs du sacrifice et du devoir :
« Grand-mère a eu plus d'impact sur ce que je suis devenu que n'importe quelle théorie de l'éducation. Elle vivait seule et était très autonome. »
Premiers pas dans le monde artistique
Il est âgé de dix ans lorsque son père trouve enfin un emploi lucratif en tant qu'assureur à la Connecticut Mutual Life Insurance Co. Mais la Guerre éclate. Clinton Sr. étant mobilisable, il devient tuyauteur sur des chantiers navals. Peu de temps après, l'économie prend un nouvel essor grâce à la Guerre, et les Eastwood en profitent. La famille achète une résidence sur la Hillside Avenue, à quelques pas de l'école de Clint Jr. L'époque de sa vie qu'il qualifiait de « merdique » est terminée.

Piedmont, ville natale de Clint Eastwood
Bien qu'appartenant à une famille tournée vers la religion, Clint Jr. n'est inscrit sur aucun registre de baptême et ne va jamais à la messe. Ce manque est certainement dû aux déménagements de son enfance. Lorsque David Frost lui demande si la religion est importante pour lui, Clint Eastwood répond : « Je ne souscris à aucune religion officielle. Mais j'ai toujours accordé beaucoup d'importance à ce genre de choses […]. Surtout quand je suis dans la nature. Je crois que c'est pour ça que j'ai tourné autant de films […] dans la nature. […] Je n'ai jamais vraiment réfléchi là-dessus à haute voix. »
Clint trouve son premier travail comme caddy sur un terrain de golf. Il distribue aussi le journal Oakland Tribune, tond des pelouses et emballe les courses des clients d'une épicerie locale pour se faire de l'argent de poche. En parallèle, sa vie scolaire n'est pas très épanouie : il change près de dix fois d'établissement. Il fréquente notamment les écoles Glenview, Crocket Highlands et Frank Havens School, toutes à proximité de Piedmont. À la deuxième d'entre elles, Eastwood suit un cours de photographie, ce qui se révèle être son premier contact avec le monde artistique. Plus tard, au collège, Clint Eastwood découvre la comédie. Bien qu'il soit introverti, il est choisi parmi tous les élèves de sa classe pour interpréter le rôle principal d'une pièce par son professeur d'anglais, Gertrude Falk. Désastreux au début, il prend peu à peu confiance en lui et termine la pièce avec plusieurs rires appréciateurs.
Malgré la présence de sa sœur, Clint joue seul. Il s'invente des amis imaginaires et mime des scènes avec ses jouets. Voici ce que Clint Eastwood déclara à ce sujet au McCall's en 1987 :
« Comme j'étais presque toujours le petit nouveau, je jouais souvent tout seul, et dans ces cas-là votre imagination devient très vite active. Vous vous inventez plein de petites histoires dans votre tête… »
Il découvre le jazz grâce à sa mère qui collectionne des disques. De son côté, son père joue de la guitare et chante dans un groupe improvisé. Clint grandit ainsi en écoutant des morceaux de jazz et de rhythm and blues. Il commence lui-même à jouer de la clarinette, puis du piano. Il finit même par prendre des cours. Cela deviendra par la suite une de ses passions.
La période « rebelle »
Clint entre à l'école secondaire en 1945. Il est indifférent à l'éducation et doit suivre les cours de rattrapage pour pouvoir passer en deuxième année. Bien élevé et socialement avantagé, Clint Eastwood devient de plus en plus un « marginal » qui cherche à se montrer rebelle. Le personnage solitaire du collège est désormais entouré de plusieurs amis. Malgré son physique sportif, Eastwood n’est pas un bon athlète, il ne s'investit pas dans les équipes sportives de l'école. Il déclare à ce sujet qu'il ne s'est « jamais vraiment impliqué dans les sports d'équipe, à cause de tous les déménagements ». Ce n'est pas exactement la vraie raison puisqu'il ne déménagea plus à partir de 1940. Les seuls sports que le futur acteur pratique sont le golf et le tennis. Il n'est pas intéressé par les sports collectifs. C'est pourtant un travail collectif qu'il effectue en créant la Malpaso des années plus tard, en embauchant de nombreuses personnes qui travailleront à ses côtés.

L'école technique d'Oakland dans lequel Eastwood termine ses études
Après avoir validé sa première année à l'école secondaire de Piedmont, Clint Eastwood la quitte pour l'école technique. Les raisons de ce départ sont assez floues. Certains affirment que c'est à cause des cours de théâtre que dispensait l'école technique que l'acteur changea d'établissement. D'autres avancent que c'est l'absence de familles noires ou asiatiques qui poussèrent Clint Eastwood à partir et d'autres déclarent qu'il a quitté l'école sur la demande de ce dernier — Clint aurait inscrit sur le panneau d'affichage du stade de l'école des propos obscènes sur l'une de ses employées.
Il finit son cursus dans cette école technique. Durant cette période, il obtient sa première voiture, alors qu'il n'avait pas l'âge légal pour la conduire. L'acteur avait deux priorités dans la vie : les voitures et les filles. Il assouvit sa passion avec ses copains, entre balades en voiture et flirt à l'arrière. On remarque d'ailleurs qu'une fois sa société de production créée, il enchaîne les films sur ces thèmes : Le Canardeur (1974), L'Épreuve de force (1977), Honkytonk Man (1982), Pink Cadillac (1989) ou encore La Relève (1990). À l'école secondaire, plutôt que de suivre des cours de théâtre, Clint assiste à des cours de mécanique et d'aéronautique. Il ne pense alors pas à son avenir, préférant vivre aux côtés de ses amis plutôt que de travailler ses leçons.
En 1948, la famille Eastwood doit à nouveau déménager, à la suite d'une promotion de Clinton Sr. Il est nommé directeur de l'une des usines de la société, à Seattle. Ses parents laissent derrière eux Clint Jr., qui termine son semestre à l'école hébergé par Harry Pendleton, l'un de ses camarades. Ainsi, à dix-neuf ans, il obtient son baccalauréat américain, malgré une scolarité dissipée,. Après ceci, Clint Eastwood demeure encore chez son camarade quelque temps. Entouré de son groupe d'amis, il est persuadé que la vie étudiante n'a aucun attrait. Il ne voit qu'un côté positif : faire la fête. Dans cette optique, il côtoie nombre de discothèques chaque fin de semaine. Un soir, alors qu'il rentre chez lui en voiture, accompagné de quelques amis, ils sont contraints de s'arrêter pour ne pas percuter des chevaux qui traversent la route. L'un d'entre eux reconnaît les chevaux : « Stop ! Je sais à qui ces chevaux appartiennent. » Tous descendent alors de la voiture, et ramènent les chevaux à leur propriétaire qui n'était autre que Howard Hawks. Eastwood croisa pour la première fois Hawks, réalisateur et producteur notamment des westerns de John Wayne. « Ce fut la seule rencontre d'Eastwood avec Howard Hawks, qui était l'un de ses réalisateurs préférés […]. Il dit considérer Hawks, de même que John Ford et Anthony Mann, comme des hommes qui ont beaucoup influencé son propre travail » écrit Janet Maslin dans un article du New York Times en 1993. Cependant, Clint n'échange aucune parole avec Hawks lors de leur rencontre.
Le début de l'âge adulte
Les prémices de la collaboration avec Universal

La Seattle University dans laquelle Clint souhaite poursuivre ses études
Au début de l'été 1949, Clint Eastwood part rejoindre sa famille à Seattle. Malgré son manque de qualifications, il se fait embaucher dans une usine de Weyerhaeuser Company à Springfield, dans laquelle il reste un an. Il enchaîne ensuite plusieurs petits travaux : il fait l'inventaire des pièces chez Boeing, conduit un camion pour Color Shake, puis est veilleur de nuit chez Bethlehem Steel. En parallèle, il suit une formation et obtient de la Croix-Rouge le diplôme de maître-nageur. Il reçoit en même temps sa convocation au service militaire, où ce diplôme se révéla précieux. Il décide alors de poursuivre des études supérieures de musique à la Seattle University. Les étudiants ne sont pas repris, à cause de l'engagement du général Lewis B. Hershey d'envoyer 30 000 hommes en quatre-vingt-dix jours en Corée. Clint fait appel auprès du conseil de révision pour obtenir un délai, mais on le lui refuse.
`

const bigStr =
`Clint Eastwood (prononcé en anglais : /klɪnt istwʊd/), né le 31 mai 1930 à San Francisco (Californie), est un acteur, réalisateur, compositeur et producteur de cinéma américain.
Autodidacte, il entre grâce à des amis au studio Universal où il interprète d’abord de petits rôles dans des séries B, puis l’un des rôles phares d'une longue série, Rawhide. Il se fait alors remarquer par Sergio Leone qui l’embauche pour la Trilogie du dollar (Pour une poignée de dollars, Et pour quelques dollars de plus et Le Bon, la Brute et le Truand). Devenu célèbre, il interprète de nombreux rôles, d’abord pour Universal, puis pour Warner Bros., notamment ceux de L'Inspecteur Harry. En 1968, il devient producteur avec la création de la société Malpaso et réalise son premier film en 1971, avec Un frisson dans la nuit. Aujourd'hui, avec plus de quatre-vingt films à son actif, parmi lesquels Impitoyable, Sur la route de Madison ou encore Mystic River et plus récemment Million Dollar Baby, Gran Torino, Invictus et American Sniper, Clint Eastwood figure parmi les cinéastes les plus reconnus au monde.
D'abord connu pour ses rôles d'antihéros volontiers redresseur de torts et tragiques, dans des films d'action violents ou des westerns tels que L'Homme des Hautes Plaines ou encore Pale Rider, il a ensuite endossé des rôles plus touchants dans des films empreints d'un certain classicisme, influencés par le cinéma de John Ford et de Howard Hawks. Il est également connu pour ses comédies telles que Doux, dur et dingue et Ça va cogner. Il a ainsi été récompensé à de nombreuses reprises, remportant notamment quatre Oscars, cinq Golden Globes, trois Césars et la Palme d'honneur au Festival de Cannes en 2009.
Étymologie du nom
Selon Le Robert des noms propres, le nom Eastwood, vient du vieil anglais qui signifie « bois (wudu) de l'Est (ēast) » ou du vieux scandinave °veit « prairie ».
Biographie
Origines

Hôpital Saint Francis où naît Clint Eastwood
Clint Eastwood a toujours été mystérieux sur ses origines, sa vie privée et son passé. Eastwood est sélectif car il veut être celui qui sait sans divulguer. Lors d'interviews, il dévoile seulement la partie de son arbre généalogique qui met en valeur son image,. Pourtant, les origines d'Eastwood suivent de près l'histoire américaine. Ses ancêtres arrivent en Amérique du Nord au milieu du xviie siècle. Ils font partie des premiers colons à se lancer dans la conquête de l'Ouest. Sa famille se partage donc entre des membres installés à New York, dans l'Ohio, dans le Michigan, en Virginie, dans l'Illinois, en Louisiane, au Kansas, dans le Colorado, le Nevada, en Californie et enfin en Alaska. Bien avant que Clint Eastwood naisse, sa famille est marquée par le monde du spectacle. Le premier Eastwood né en Amérique est Lewis Eastwood. Ses parents sont venus d'Angleterre ; ils sont toutefois d'origine irlandaise,. Bien qu'il ait déclaré à la presse être « le premier de la famille à avoir réussi », Clint Eastwood est bien loin de la vérité : à la fin du xviiie siècle, Lewis Eastwood est devenu un entrepreneur renommé, classé cent troisième parmi les mille deux cents commissionnaires de la ville de New York. L'un des petits-fils de Lewis, Asa Bedesco Eastwood, l'arrière-grand-père de Clint, quitte la ville pour devenir mineur. C'est à cause de lui que le réalisateur a souvent montré, à travers ses films, une tendresse particulière à l'égard des mineurs, comme dans La Kermesse de l'Ouest (1969), L'Homme des Hautes Plaines (1973) ou encore dans Pale Rider, le cavalier solitaire (1985). Habitué du commerce, Burr, l'un des fils d'Asa Bedesco, quitte sa famille pour travailler comme magasinier, emploi dans lequel il monte rapidement l'échelle sociale. Il épouse Jessie Anderson, une immigrante d'Écosse, qui lui donne deux fils, dont Clinton. Clinton se marie en 1927 avec Margaret Ruth Runner, une femme de la haute société. Ils donnent naissance à un garçon qui leur dédie plus tard l'Oscar du meilleur film qu'il remporte pour Impitoyable,, :
« Cette victoire est simplement merveilleuse, je la décerne à toutes les personnes auxquelles je pourrais penser. […] Durant cette année de la femme, la plus grande femme sur la planète est ici ce soir — ma mère, Ruth. »
Clinton Eastwood Jr. est donc né le 31 mai 1930, à l'hôpital Saint Francis de San Francisco. À cette époque, le nourrisson était déjà célèbre. Sa mère déclare au journal anglais News of the World : « C'était le plus gros bébé de la maternité. Il pesait 5,2 kg. Les infirmières s'amusaient beaucoup à le montrer aux autres mamans. Elles l'appelaient « Samson ». Il était tellement grand,. » Il fut surnommé Clinton Jr. en hommage à son père, bien que son nom complet soit Clinton Elias Eastwood Jr. Toutefois, il fut surnommé par ses parents « Sonny ». Les liens familiaux sont forts chez les Eastwood, comme l'exprime Ruth lors de la naissance de son fils : « Je suis tombée amoureuse de lui dès qu'il est né ! ». Et cet amour est largement restitué dans tous les films dans lesquels Clint Jr. est impliqué par la suite.
Enfance
La Grande Dépression

Oakland, où Eastwood aurait passé son enfance.
Ses différents attachés de presse ont, durant quarante ans, clamé que Clint Eastwood était originaire d'Oakland, ville ouvrière qui mettrait en valeur la réussite d'Eastwood. Ce dernier a même déclaré dans une interview que s'il traitait si souvent les gens de « trous du cul » dans ses films, c'était probablement à cause de son enfance passée à Oakland. Mais cette information n'est pas vraiment exacte. Dans la biographie écrite par Schickel publiée en 1996, on découvre qu'Eastwood a, en fait, grandi à Piedmont,. Schickel y déclare que les Eastwood ont grandi dans une « modeste maison au toit couvert de bardeaux », mais il précise que « cette maison était [toutefois] située à la limite d'Oakland ». L'enfance de Clint Eastwood est marquée par la Grande Dépression et le passage au cinéma sonore. Les journaux locaux ne traitent guère de la crise. Toutefois le chômage ne cesse d'augmenter. Il atteint un taux de 28 % en Californie. Si Oakland, d'origine ouvrière, est très touchée par la Grande Dépression, Piedmont fait figure de banlieue chic où la crise n'a pas de réel impact sur la vie de tous les jours. Toutefois, les parents du jeune Eastwood quittent la région : Clinton Sr. vient de perdre son emploi de commercial chez East Bay Refrigeration Products,.

Spokane, ville dans laquelle Clinton Sr. trouve un travail temporaire.
Selon les divers témoignages, Clinton Sr. se met en quête de travail partout où il y en a. Il déclare ainsi à son fils : « dans la vie, on n’a rien pour rien », ce que Clint Jr. n’a jamais contesté. C'est d'ailleurs peut-être de ce nomadisme que naît la future passion d'Eastwood pour les westerns. Il n'a ni diplôme universitaire ni qualification professionnelle. Les voyant découragés, le frère de Ruth, la mère de Clint Jr., les dépanne financièrement comme il peut. Il aide par ailleurs Clinton Sr. à trouver un emploi dans une usine de réfrigérateurs à Spokane. Ce dernier enchaîne avec un travail de pompiste sur Sunset Boulevard qu'il obtient grâce à des amis. La famille s'installe alors à Pacific Palisades, un district de Los Angeles. C'est durant cette période que Clint Jr. manque de mourir noyé à l'âge de quatre ans et qu'il assiste à la naissance de sa sœur, Jeanne. L'enfance de Clint Jr. est ainsi marquée par des déménagements incessants dus aux changements de travail de son père : ils vont notamment à Sacramento, et Redding. Ces voyages vont durer près de six ans. Cependant, Schickel déclare dans son livre sur Clint Eastwood qu'« il n'y avait jamais ni panique ni désespoir dans ces déménagements. […] Quand la famille faisait ses paquets, M. Eastwood avait toujours retrouvé un emploi. Et à aucun moment Clint ne s'est senti délaissé ou abandonné durant cette période. ». Au milieu des années 1930, la mère de Clint Eastwood achète la maison de sa tante à Piedmont pour une somme dérisoire. En évoquant cette période, l'acteur déclare au Village Voice, en 1976, que « c'était une époque merdique ». Il ajoute « on n'était pas itinérants. […] C'était pas Les Raisins de la colère, mais c'était pas le luxe non plus » au Rolling Stone.
De retour dans sa ville natale, Clint Eastwood rend souvent visite à sa grand-mère, Virginia May Runner, jusqu'en 1937, date à laquelle cette dernière déménage vers la région rurale, derrière les faubourgs Est d'Oakland. Malgré son départ, Clint Eastwood ne la perd pas de vue pour autant, il va chez elle de temps en temps. C'est durant ces quelques séjours que Clint Eastwood apprend à monter à cheval. Il y apprend également les valeurs du sacrifice et du devoir :
« Grand-mère a eu plus d'impact sur ce que je suis devenu que n'importe quelle théorie de l'éducation. Elle vivait seule et était très autonome. »
Premiers pas dans le monde artistique
Il est âgé de dix ans lorsque son père trouve enfin un emploi lucratif en tant qu'assureur à la Connecticut Mutual Life Insurance Co. Mais la Guerre éclate. Clinton Sr. étant mobilisable, il devient tuyauteur sur des chantiers navals. Peu de temps après, l'économie prend un nouvel essor grâce à la Guerre, et les Eastwood en profitent. La famille achète une résidence sur la Hillside Avenue, à quelques pas de l'école de Clint Jr. L'époque de sa vie qu'il qualifiait de « merdique » est terminée.

Piedmont, ville natale de Clint Eastwood
Bien qu'appartenant à une famille tournée vers la religion, Clint Jr. n'est inscrit sur aucun registre de baptême et ne va jamais à la messe. Ce manque est certainement dû aux déménagements de son enfance. Lorsque David Frost lui demande si la religion est importante pour lui, Clint Eastwood répond : « Je ne souscris à aucune religion officielle. Mais j'ai toujours accordé beaucoup d'importance à ce genre de choses […]. Surtout quand je suis dans la nature. Je crois que c'est pour ça que j'ai tourné autant de films […] dans la nature. […] Je n'ai jamais vraiment réfléchi là-dessus à haute voix. »
Clint trouve son premier travail comme caddy sur un terrain de golf. Il distribue aussi le journal Oakland Tribune, tond des pelouses et emballe les courses des clients d'une épicerie locale pour se faire de l'argent de poche. En parallèle, sa vie scolaire n'est pas très épanouie : il change près de dix fois d'établissement. Il fréquente notamment les écoles Glenview, Crocket Highlands et Frank Havens School, toutes à proximité de Piedmont. À la deuxième d'entre elles, Eastwood suit un cours de photographie, ce qui se révèle être son premier contact avec le monde artistique. Plus tard, au collège, Clint Eastwood découvre la comédie. Bien qu'il soit introverti, il est choisi parmi tous les élèves de sa classe pour interpréter le rôle principal d'une pièce par son professeur d'anglais, Gertrude Falk. Désastreux au début, il prend peu à peu confiance en lui et termine la pièce avec plusieurs rires appréciateurs.
Malgré la présence de sa sœur, Clint joue seul. Il s'invente des amis imaginaires et mime des scènes avec ses jouets. Voici ce que Clint Eastwood déclara à ce sujet au McCall's en 1987 :
« Comme j'étais presque toujours le petit nouveau, je jouais souvent tout seul, et dans ces cas-là votre imagination devient très vite active. Vous vous inventez plein de petites histoires dans votre tête… »
Il découvre le jazz grâce à sa mère qui collectionne des disques. De son côté, son père joue de la guitare et chante dans un groupe improvisé. Clint grandit ainsi en écoutant des morceaux de jazz et de rhythm and blues. Il commence lui-même à jouer de la clarinette, puis du piano. Il finit même par prendre des cours. Cela deviendra par la suite une de ses passions.
La période « rebelle »
Clint entre à l'école secondaire en 1945. Il est indifférent à l'éducation et doit suivre les cours de rattrapage pour pouvoir passer en deuxième année. Bien élevé et socialement avantagé, Clint Eastwood devient de plus en plus un « marginal » qui cherche à se montrer rebelle. Le personnage solitaire du collège est désormais entouré de plusieurs amis. Malgré son physique sportif, Eastwood n’est pas un bon athlète, il ne s'investit pas dans les équipes sportives de l'école. Il déclare à ce sujet qu'il ne s'est « jamais vraiment impliqué dans les sports d'équipe, à cause de tous les déménagements ». Ce n'est pas exactement la vraie raison puisqu'il ne déménagea plus à partir de 1940. Les seuls sports que le futur acteur pratique sont le golf et le tennis. Il n'est pas intéressé par les sports collectifs. C'est pourtant un travail collectif qu'il effectue en créant la Malpaso des années plus tard, en embauchant de nombreuses personnes qui travailleront à ses côtés.

L'école technique d'Oakland dans lequel Eastwood termine ses études
Après avoir validé sa première année à l'école secondaire de Piedmont, Clint Eastwood la quitte pour l'école technique. Les raisons de ce départ sont assez floues. Certains affirment que c'est à cause des cours de théâtre que dispensait l'école technique que l'acteur changea d'établissement. D'autres avancent que c'est l'absence de familles noires ou asiatiques qui poussèrent Clint Eastwood à partir et d'autres déclarent qu'il a quitté l'école sur la demande de ce dernier — Clint aurait inscrit sur le panneau d'affichage du stade de l'école des propos obscènes sur l'une de ses employées.
Il finit son cursus dans cette école technique. Durant cette période, il obtient sa première voiture, alors qu'il n'avait pas l'âge légal pour la conduire. L'acteur avait deux priorités dans la vie : les voitures et les filles. Il assouvit sa passion avec ses copains, entre balades en voiture et flirt à l'arrière. On remarque d'ailleurs qu'une fois sa société de production créée, il enchaîne les films sur ces thèmes : Le Canardeur (1974), L'Épreuve de force (1977), Honkytonk Man (1982), Pink Cadillac (1989) ou encore La Relève (1990). À l'école secondaire, plutôt que de suivre des cours de théâtre, Clint assiste à des cours de mécanique et d'aéronautique. Il ne pense alors pas à son avenir, préférant vivre aux côtés de ses amis plutôt que de travailler ses leçons.
En 1948, la famille Eastwood doit à nouveau déménager, à la suite d'une promotion de Clinton Sr. Il est nommé directeur de l'une des usines de la société, à Seattle. Ses parents laissent derrière eux Clint Jr., qui termine son semestre à l'école hébergé par Harry Pendleton, l'un de ses camarades. Ainsi, à dix-neuf ans, il obtient son baccalauréat américain, malgré une scolarité dissipée,. Après ceci, Clint Eastwood demeure encore chez son camarade quelque temps. Entouré de son groupe d'amis, il est persuadé que la vie étudiante n'a aucun attrait. Il ne voit qu'un côté positif : faire la fête. Dans cette optique, il côtoie nombre de discothèques chaque fin de semaine. Un soir, alors qu'il rentre chez lui en voiture, accompagné de quelques amis, ils sont contraints de s'arrêter pour ne pas percuter des chevaux qui traversent la route. L'un d'entre eux reconnaît les chevaux : « Stop ! Je sais à qui ces chevaux appartiennent. » Tous descendent alors de la voiture, et ramènent les chevaux à leur propriétaire qui n'était autre que Howard Hawks. Eastwood croisa pour la première fois Hawks, réalisateur et producteur notamment des westerns de John Wayne. « Ce fut la seule rencontre d'Eastwood avec Howard Hawks, qui était l'un de ses réalisateurs préférés […]. Il dit considérer Hawks, de même que John Ford et Anthony Mann, comme des hommes qui ont beaucoup influencé son propre travail » écrit Janet Maslin dans un article du New York Times en 1993. Cependant, Clint n'échange aucune parole avec Hawks lors de leur rencontre.
Le début de l'âge adulte
Les prémices de la collaboration avec Universal

La Seattle University dans laquelle Clint souhaite poursuivre ses études
Au début de l'été 1949, Clint Eastwood part rejoindre sa famille à Seattle. Malgré son manque de qualifications, il se fait embaucher dans une usine de Weyerhaeuser Company à Springfield, dans laquelle il reste un an. Il enchaîne ensuite plusieurs petits travaux : il fait l'inventaire des pièces chez Boeing, conduit un camion pour Color Shake, puis est veilleur de nuit chez Bethlehem Steel. En parallèle, il suit une formation et obtient de la Croix-Rouge le diplôme de maître-nageur. Il reçoit en même temps sa convocation au service militaire, où ce diplôme se révéla précieux. Il décide alors de poursuivre des études supérieures de musique à la Seattle University. Les étudiants ne sont pas repris, à cause de l'engagement du général Lewis B. Hershey d'envoyer 30 000 hommes en quatre-vingt-dix jours en Corée. Clint fait appel auprès du conseil de révision pour obtenir un délai, mais on le lui refuse.
Clint arrive en 1950 à Fort Ord, le centre de réception des appelés, où des milliers de jeunes recrues arrivent pour renforcer l'armée du général Douglas MacArthur, qui souhaite mener une offensive vers le nord de la Corée. Son diplôme de maître-nageur lui vaut de ne pas partir en Corée, mais de devenir professeur de natation au camp,. Il n'est pas envoyé en Corée grâce à la qualité de ses cours pour laquelle il termine caporal, et fait même l'objet d'une citation récompensant son mérite.
En sa qualité d'enseignant militaire, il lui est nécessaire de faire preuve de sang-froid et de témoigner d'un esprit de commandement qui lui sert par la suite, quand il devient réalisateur. Clint Eastwood déclare qu'une équipe de tournage, « C'est comme un peloton. Je guide le peloton vers l'endroit où il doit aller. » Fort Ord ressemble à une vraie ville : outre la caserne, on y trouve un centre de sport, une cantine, un hôpital, des magasins, des théâtres et des cinémas. Universal Pictures semble avoir entretenu une grande relation avec Fort Ord. Les nouveaux films y sont souvent montrés avant leur sortie nationale ; leur projection bénéficie même de la présence des acteurs et réalisateurs. Clint passe ainsi ses deux années de service, sans toutefois réellement entrer en relation avec une quelconque célébrité du monde du cinéma ; il réussit pourtant à s'ouvrir les portes des studios Universal.

Fort Ord durant un entraînement
En 1952, Clint Eastwood peut voter pour la première fois. À l'instar de toute sa famille, il s'oriente vers le Parti républicain et vote pour Dwight David Eisenhower,. Il est entré en contact avec Universal International durant son service militaire, mais la manière dont cela s'est déroulé est assez floue. Plusieurs théories ont été proposées, et personne ne peut dire laquelle est la bonne. La première d'entre elles a été publiée dans un communiqué de presse publicitaire du groupe, le 18 février 1955 : on y apprend que Clint a été découvert par un individu en visite au Fort Ord qui a remarqué son physique avantageux. « Clint Eastwood a été découvert par le réalisateur Arthur Lubin durant le tournage de Francis chez les wacs à Fort Ord,,. » Le communiqué de CBS lors de la sortie de Rawhide est plus complet à ce sujet : « une équipe de tournage Universal International était en train de travailler à Fort Ord, en Californie. Un audacieux assistant-réalisateur remarqua le beau jeune homme de 1,95 mètre alors qu'il s'apprêtait à faire la queue pour la cantine. Il lui dit : « quand tu auras fini, passe faire un tour sur le plateau. Je voudrais que tu rencontres notre réalisateur ». Clint s'exécuta et le réalisateur fut tellement impressionné par son physique […] qu'il lui demanda de le rappeler à Universal dès qu'il aurait terminé son service,. La deuxième théorie au sujet de cette rencontre est légèrement différente. Publiée par Schickel dans son livre, elle met en avant Chuck Hill, une recrue de Fort Ord, qui encourage Clint à se rendre à Los Angeles. Mais Clint ne fait rien. Les deux hommes restent en contact, et Hill obtient un jour un poste à Universal où il fait entrer en cachette son ami. Il le présente à un caméraman, Irving Glassberg, qui voit en lui la future vedette,. La troisième théorie est avancée par Earl Leaf. Ce dernier affirme que Clint restait durant des heures assis sur un tabouret en espérant se faire remarquer, à l'image de Lana Turner, découverte sur un tabouret du bistro Schwab's. Et, un jour ses espoirs se réalisent lorsqu'il rencontre une jeune standardiste qui le fait entrer à Universal. Il semble que la première théorie, bien que déformée, se rapproche le plus de la réalité.
La chance de Clint
Plus tard, Clint Eastwood quitte Seattle, où il a mis enceinte une fille dont les parents fréquentaient les siens. Scandalisés, ils fournissent à Clint la somme nécessaire pour payer l'avortement de la jeune fille, malgré le fait qu'il propose de l'épouser. Il promet alors à ses parents de devenir plus sérieux. Plus tard, il explique à ses amis que cet épisode fut « dévastateur » pour lui, que cette fille reste son seul « véritable amour ». Il décide donc de partir pour Los Angeles où il reprend ses études au Los Angeles City College et se met à fréquenter une fille qu'il avait rencontrée et fréquentée durant son service à Ord : Margaret Neville Johnson, surnommée « Maggie ». Elle y travaillait comme secrétaire pour l'Industria Americana,,,. Un an plus tard, le couple annonce ses fiançailles. Et, lors de Noël 1953, à South Pasadena, les deux amoureux se marient.
Le Los Angeles City College est considéré comme le meilleur établissement de la ville pour apprendre la comédie, il a notamment formé Kim Novak, Robert Vaughn ou encore James Coburn. D'ailleurs, beaucoup de studios y envoient leurs acteurs sous contrat, pour qu'ils poursuivent leur formation. Malgré cette réputation, Clint Eastwood ne va pas dans cette université pour suivre des cours d'art dramatique, mais pour y suivre une formation commerciale. Il étudie ainsi de septembre 1953 à février 1954. Mais au printemps, Eastwood décide d'abandonner ses études : en avril, grâce à des personnes rencontrées durant son service militaire, il est embauché chez Universal, où il signe un contrat de courte durée. Malgré la récession qui sévit aux États-Unis, Universal semble s'en sortir en produisant de nombreux films à petit budget. Clint Eastwood est donc embauché comme « inconnu pas cher », avec un salaire de 75 $ par semaine. Mais à cette époque rien n'est encore gagné, Clint n'ayant jamais appris à jouer la comédie.
En 1950, cependant, Sophie Rosenstein crée la Universal Talent School où l'on apprend la comédie. Chaque année, plus de soixante personnes s'y présentent, dix seulement gagnent le droit de passer une audition et deux ou trois sont retenues pour faire un essai filmé. Le premier critère de sélection, à l'époque, est le physique. En rencontrant Clint, Arthur Lubin a déclaré qu'« il était tellement grand, mince et beau ». Il lui propose immédiatement de faire un essai filmé, mais Eastwood, n'ayant aucune expérience du métier d'acteur, ne sait pas où se positionner ni ce qu'il doit faire. Malgré cet essai décevant, Lubin lui affirme qu'« il faut persévérer ». Je te conseille d'aller à l'école d'art dramatique du studio ». C'est ainsi qu'Eastwood obtient son contrat avec la société Universal. Signé le 26 avril 1954, il stipule que le studio bénéficie de ses « services exclusifs à titre d'artiste pour ce qui est du cinéma, des apparitions personnelles et des productions théâtrales, radiophoniques et télévisuelles ». Le contrat dure vingt semaines, avec un salaire de 100 $ par semaine et la possibilité d'être prolongé.
Clint se montre un bon élève dès les premières semaines : s'il n'a pas toujours de bonnes notes, il est consciencieux et attentif, ce que relèvent les professeurs qui le considèrent comme l'un de leurs meilleurs élèves. Toutefois, sa réussite se limite aux cours ; lorsque Eastwood joue, il demeure froid et rigide. D'ailleurs, lorsqu'il passe sa première audition pour jouer dans le film La police était au rendez-vous (Six Bridges to Cross) de Joseph Pevney en mai 1954, il n'obtient aucun rôle. Il tente, sans succès, de jouer des scènes tirées de Brigadoon, Tessa, La Nymphe au cœur fidèle ou encore de Sept ans de réflexion pour montrer aux directeurs de casting ce qu'il vaut. Alors, il se rabat sur le doublage. Il travaille ainsi sur La Révolte des Cipayes, Le Signe du païen, Le Fleuve de la dernière chance et sur Deux nigauds et les flics.
Une carrière naissante
L'ayant remarqué à Universal lorsque Lubin travaillait sur son film Francis chez les wacs, Jack Arnold décide d'engager Eastwood pour les besoins du tournage de La Revanche de la créature. Il y joue le petit rôle d'un laborantin, Jennings, qui assiste un médecin (John Agar) qui mène des recherches sur un monstre,. Durant les années 1950, il obtient plusieurs rôles, mais toutes ses apparitions sont insignifiantes pour l'intrigue. C'est ainsi que le jeune Eastwood et sa femme, Maggie, déménagent dans un appartement à la Villa Sands, sur Ventura Boulevard, pour être plus proches des studios Universal. Ils y côtoient des jeunes célébrités telles que Gia Scala, Anita Ekberg ou encore Lili Kardell. Eastwood est alors un ami proche de Scala et de Kardell, toutes deux également comédiennes de la Talent School. La période rebelle est oubliée, Clint essaye désormais de réussir sa vie.

Entrée des studios Universal où Eastwood suit ses cours
Clint Eastwood apparaît à l'écran moins souvent que ses collègues de la Talent School. On le voit dans quelques films, mais sa présence est à peine remarquable. On le retrouve dans Ne dites jamais adieu, Brisants humains ou encore dans Les Piliers du ciel et La corde est prête. Il participe, sans être toutefois crédité, aux films Le Cavalier au masque, El Tigre, La Danseuse et le Milliardaire (Ain't Misbehavin'), Les Forbans, L'Enfer des hommes, La Jungle des hommes, Coup de fouet en retour (Backlash) et Benny Goodman. Si Universal profite ainsi de lui, Clint en tire profit et observe durant son apprentissage toutes les étapes de la fabrication d'un film.
Mais, en septembre 1955, son contrat avec Universal est sur le point d'expirer. Eastwood est persuadé que la société le renouvellera. Aussi, en rentrant des deux semaines de vacances qui lui avaient été accordées avec sa femme, il est face à une désillusion : son contrat et celui de deux autres personnes n'ont pas été reconduits. Cet échec renforce sa détermination à continuer sa carrière dans le cinéma. Son amitié avec Lubin demeure inchangée : celui-ci l'invite souvent à manger ou à voyager avec lui, il lui offre des costumes ou lui prête de l'argent. Le réalisateur étant homosexuel, certains pensent même qu'Eastwood l'est également. Sa femme, jalouse de cette relation, demande à Clint de ne plus jamais revoir Lubin. Toutefois, les deux hommes restent en contact. Lubin offre à Eastwood le plus grand rôle de sa carrière à l'époque, et sa première apparition au générique : celui d'un officier qui recrute pour la brigade des Rough Riders dans La VRP de choc (First Traveling Saleslady).
Il enchaîne avec un petit rôle, toujours pour Lubin, comme pilote dans Escapade au Japon (Escapade in Japan) et des apparitions à la télévision. Eastwood essaie en vain d'obtenir un contrat avec la Warner Bros., avec la Paramount Pictures ou encore avec la 20th Century Fox. C'est en 1959 qu'il réussit enfin à obtenir un grand rôle, dans la série télévisée Maverick. Il interprète le rôle d'un méchant qui essaye d'épouser une fille riche pour son argent. Toutefois, il est loin de s'épanouir grâce à son travail ; c'est Maggie Johnson, sa femme, qui, grâce à son emploi comme mannequin, permet à la famille de subvenir à ses besoins.
Il obtient en revanche une place dans C'est la guerre (Lafayette Escadrille) de William A. Wellman et un rôle, bien plus important que le précédent, dans Ambush at Cimarron Pass, western réalisé par Jodie Copelan. Il y incarne un soldat sudiste qui explore la frontière à la recherche de trafiquants d'armes. Considérant Ambush at Cimarron Pass comme la « pire étape de sa carrière », et abattu par le manque de succès, il est prêt à abandonner le cinéma. Lorsqu'il assiste à une projection du film avec sa femme, il lui déclare : « Je vais arrêter. Il faut vraiment que j'arrête. Il faut que je retourne à l'école. Je dois commencer à faire quelque chose de ma vie ». Après avoir été brièvement sous contrat avec la Marsh Agency, il trouve un nouvel agent, Bill Shiffrin. C'est la signature de ce contrat qui est certainement décisive dans la carrière de Clint Eastwood à cette époque. Allant plus loin dans la collaboration que Lubin, Shiffrin va permettre à Eastwood de se distinguer, et ce, à la télévision.
L'âge adulte : Universal et United Artists
Un succès imminent

Eastwood et Don Hight durant le tournage de Rawhide.
Shiffrin remarque la carrure d'Eastwood et l'estime parfait pour un casting dont il a entendu parler. Ce casting est organisé par CBS Corporation pour les besoins d'un feuilleton, un western diffusé en épisodes d'une heure. En entrant dans les locaux de la société, un cadre (qui n'était autre que Robert Sparks) le remarque et lui demande : « Combien mesurez-vous ? ». À quoi Eastwood répond « 1,95 mètre ». Le cadre l'invite alors à le suivre dans son bureau. Eastwood y rencontre pour la première fois Charles Marquis Warren, le producteur de la série. Le lendemain, son agent lui annonce qu'il doit passer des essais : lire un monologue d'Henry Fonda issu de L'Étrange Incident. Clint pense avoir tout loupé ; pourtant, une semaine plus tard, Shiffrin le contacte pour lui annoncer qu'il a obtenu le rôle. Il vient d'obtenir son rôle le plus important à l'époque, celui d'un cow-boy nommé Rowdy. La série traite de la transhumance, c'est pourquoi Warren voulait la nommer Outrider,, mais la direction de la chaîne préfère Rawhide, titre définitif de la série. Si le tournage débute bel et bien, sa programmation reste incertaine, car CBS ne sait pas encore comment l'introduire dans son programme, comment la mettre en valeur. Clint se souvient avoir pensé que sa « carrière va s'arrêter là, dans un sous-sol, dans un tiroir à CBS ». À Noël, Eastwood et sa femme partent voir la famille de Clint à Piedmont. C'est alors qu'ils reçoivent un télégramme annonçant la diffusion imminente de Rawhide, et la reprise du tournage dès le mois suivant.

Nogales, où a lieu le tournage de Rawhide
Eric Fleming interprète le premier rôle de Rawhide, mais pourtant Eastwood se considère comme la vedette, il en parle comme de « sa série » à ses amis. Les deux acteurs, lors du premier jour de tournage, en Arizona, en viennent même aux mains. Toutefois, Eastwood, en public, marque toujours un certain respect pour Fleming. Par ailleurs, Eastwood est considéré sur le plateau de tournage comme un proche par beaucoup d'artistes, comme Charles Marquis Warren et Paul Brinegar, qui l'identifient à leur propre frère. Et cela tout au long du tournage, où le personnage de Fleming évoque la compassion d'un grand frère envers son petit frère ; Eastwood incarne l'homme fougueux qui n'a pas encore suffisamment d'expérience. Finalement, au fil du temps, le personnage incarné par Fleming perd en importance au profit de celui d'Eastwood. Cela est encore plus marqué dans le premier épisode de la deuxième saison, quand Endre Bohem reprend la série après le départ de Warren. Pour la première fois l'épisode est présenté par Rowdy, qui déclare « je suis Rowdy Yates, bouvier de cette bande… ».
Si Fleming demeure la star de la série, Eastwood monte dans l'estime des gens, et son nom commence à être connu. Son salaire s'élève désormais à 750 $ par épisode, lui permettant de quitter la Villa Sands pour une maison à Sherman Oaks. Il incarne pour l'Amérique le fils idéal, le « petit ». Ce qui ne plait pas trop à Eastwood. En effet, Rowdy incarne le « jeunot », l'adolescent, alors que Clint va avoir trente ans en 1960 ; d'ailleurs, le nom « Rowdy » peut se traduire par une personne turbulente, chahuteuse. Clint le surnomme « le Crétin des plaines »,. Son salaire lui permet d'investir : il achète nombre de voitures et de propriétés, telles que « Mal Paso » et une autre près de Monterey. Grâce à Rawhide, Clint Eastwood réalise sa première interview en 1959 :
« Il faut toujours se vendre. Il faut vanter partout les mérites de ce produit que l'on est. Il faut croire en soi de la même façon qu'un VRP croit en son aspirateur. C'est difficile, mais si vous ne le faites pas, personne ne peut savoir ce que vous valez. À Hollywood, on ne peut se permettre d'être humble que quand on est déjà devenu une star. »
En plus de son jeu d'acteur, le producteur de la série demande à Clint Eastwood de jouer de la guitare et d'interpréter A Drover's Life ; et, dans un autre épisode, de monter sur les planches d'un saloon et de chanter Beyond the Sun. Si le jazz avait bercé son enfance, c'est désormais la country qui l'intéresse. Il lance ainsi sa carrière musicale. En 1959, Clint enregistre son premier album sous le label Cameo qu'il intitule Cowboy Favorites. Toutefois, le succès de l'album est très limité. Il s'essaie à plusieurs autres reprises dans la musique, mais ses tentatives sont relativement mal accueillies.

Photographie publicitaire de Clint pour la série Rawhide
Post-Rawhide
Dès la troisième saison de Rawhide, les journaux de Hollywood relatent à quel point Eastwood est l'as de la série. Depuis la signature du contrat, CBS empêche l'acteur d'accepter une quelconque apparition dans une autre production. Dans une interview publiée dans le Hollywood Reporter, Clint déclare : « Je me prépare à me faire renvoyer, ce qui signifie que je ne pourrai plus travailler ici, mais j'ai reçu des propositions de Londres et de Rome pour des films qui devraient me rapporter plus d'argent en une année que ce que j'ai touché pour Rawhide en trois ans. » Toutefois, selon l'agent actuel de l'acteur, Ruth Marsh, Eastwood mentait. Il n'avait reçu aucune proposition, et son interview n'avait qu'un but : une augmentation de salaire. Les seules demandes qu'il a reçues se limitent à des courtes apparitions dans des programmes télévisés.
En 1964, la série quitte le Top 25 sur lequel elle trônait depuis les débuts de sa diffusion,. Endre Bohem quitte la série pour la laisser à Vincent M. Fennelly, qui la cède lui-même à Bernard L. Kowalski et Bruce Geller ; Eric Fleming aussi est de plus en plus souvent absent. Tout est mis en œuvre pour pallier cet insuccès soudain, mais cela se révèle infructueux. La série perd peu à peu sa cohésion. Fleming se voit alors proposer un rôle dans un western italien tourné en Espagne : Pour une poignée de dollars. Mais le salaire prévu n'est pas élevé et Fleming essaie de négocier une augmentation. Il finit par refuser l'offre, préférant voir son nom associé à de grosses productions hollywoodiennes. Par le biais de Irving L. Leonard, Fleming passe la proposition à Clint. Au début, Eastwood a la même réaction que Fleming : il ne veut pas s'embêter avec un petit rôle dans un western étranger ; pire, il refuse de lire le scénario. Mais, encouragé par Irving, il s'exécute en fin de compte. Il remarque une « intrigue intelligemment construite », ainsi qu'une similitude avec Le Garde du corps d'Akira Kurosawa. Eastwood se décide finalement à postuler pour le rôle de l'homme sans nom. Les producteurs, qui ne connaissent pas Clint Eastwood, sollicitent l'avis de Richard Harrison, un acteur américain installé en Italie et qui avait lui-même refusé le rôle : il leur confirme qu'Eastwood serait un interprète convaincant. Eastwood est finalement embauché pour tenir le rôle principal de Pour une poignée de dollars et signe le contrat pour 15 000 $,,. À la fin du tournage, durant le montage, Sergio Leone, le réalisateur, ne sait pas encore ce que va donner le film (il n'est pas encore le grand réalisateur qu'il deviendra) et il ne se fait aucune illusion sur le rendu final.

Eastwood dans Pour une poignée de dollars en 1964
Tandis que la septième saison de Rawhide est diffusée, en 1965, Leone termine le montage de son film. Lorsqu'il montre le résultat final à ses associés, l'un d'eux, Duccio Tessari, lui déclare qu'il s'agit d'un « très bon film ». Le titre est modifié : il ne s'agit plus de The Magnificent Stranger mais de Pour une poignée de dollars (Per un pugno di dollari). Pour la promotion du film, les producteurs tendent à montrer une œuvre américaine. Leone prend le pseudonyme de Bob Robertson, Ennio Morricone, le compositeur, devient Dan Savio et Gian Maria Volontè John Wells. Tout est fait pour que le film réussisse. Pourtant, lorsque Leone se rend au marché du film de Sorrente, aucun grand distributeur ne veut prendre de risque pour un western réalisé par un inconnu. Le film n'est finalement projeté que dans une seule salle. Après deux très mauvais jours, le film fait salle pleine. En Italie, le film rapporte finalement 3 000 000 000 ₤. Les critiques, italienne comme américaine, sont très élogieuses. Finalement nommé A Fistful of Dollars aux États-Unis, le film est vu comme le succès inattendu de l'année. Eastwood devient alors une véritable star en Italie.
« L'Homme sans nom »
Article détaillé : Trilogie du dollar.
Insérer un paragraphe

Le Bon, la Brute et le Truand
Fort de son succès, Leone propose à Eastwood un rôle dans une suite de son film. Le budget alloué augmente largement. Leone engage Luciano Vincenzoni pour l'écriture du scénario. Le film s'intitule finalement Et pour quelques dollars de plus (Per qualche dollaro in più). Pour les besoins du tournage, un nouvel acteur est embauché : Lee Van Cleef. Par ailleurs, Sophia Loren vient de proclamer Eastwood « la plus grande star masculine d'Italie ». Si le premier volet a consolidé l'image d'Eastwood en Amérique, ce deuxième lui donne une certaine notoriété mondiale. C'est la United Artists qui décide de distribuer le film à l'étranger. Les distributeurs sont enthousiastes alors qu'ils sortent d'une projection à Rome. Ils achètent le film pour 900 000 $. Alors qu'ils signent le contrat, Arnold Picker, intéressé par la production de son prochain film, demande à Sergio Leone ce qu'il compte faire par la suite. Leone n'avait jusque là rien en tête, mais il lui vient l'idée d'Il Buono, il brutto, il cattivo. Enchanté, Picker lui demande combien coûtera le film et décide de le produire. Avec Eastwood, ils tournent finalement Le Bon, la Brute et le Truand. Entre temps, Dino De Laurentiis engage Clint pour un petit rôle dans un film à sketchs, Les Sorcières (Le Streghe), sous la direction de Vittorio De Sica. Durant le tournage de Le Bon, la Brute et le Truand, Eastwood gagne en assurance. Il s'affirme au sein de l'équipe de tournage. Il parle même de se mettre prochainement à son compte pour tourner ses propres films, en tant que producteur ou réalisateur. Pourtant, sa collaboration avec Leone est des plus fructueuses : Le Bon, la Brute et le Truand est le volet le plus rémunérateur de la trilogie, il rapporte 8 000 000 $ en coûts de location,.
Toutefois, le succès de la trilogie est surtout européen. Aux États-Unis, elle est quelque peu critiquée, et Eastwood a du mal à se faire embaucher. Les attachés de presse de la United Artists déclarent que « les producteurs américains faisaient désormais la queue pour s'offrir les services de Clint », mais c'est loin d'être le cas. On lui propose un jour Pendez-les haut et court (Hang 'Em High), écrit par Mel Goldberg et Leonard Freeman. Il se montre réticent, mais face à une forte insistance de Leonard Irving, il accepte. Le film est une coproduction d'United Artists et Malpaso, société créée par Eastwood lui-même. Il est le principal actionnaire de la firme et a, par ce biais, un certain contrôle sur les films dans lesquels il apparaît : choix du script, des principaux acteurs et du réalisateur. C'est ainsi que Ted Post, un vieil ami de Clint, assure la réalisation de Pendez-les haut et court : « C'était lui qu'il voulait, se souvient Goldberg, parce qu'il y avait beaucoup de scènes de dialogue dans lesquelles il allait être obligé de jouer. Il a dit que le peu de scènes qu'il avait pris plaisir à faire dans Rawhide avaient été celles que Ted Post avait réalisées. »
Pendez-les haut et court est le quatrième western dans lequel Eastwood apparaît. Cependant, c'est le premier à être véritablement apprécié par la critique. Le New York Post déclare qu'il s'agit d'un « western de qualité, plein de courage, de périls et de passion ». Le film est également une réussite au box-office : lors de son premier jour, en juillet 1968, le film rapporte 5 241 $, ce qui se révèle être la meilleure première de toute l'histoire d'United Artists à l'époque, y compris les James Bond. En deux semaines, le film est déjà rentable pour les sociétés de production. Variety rapporte qu'United Artists considère le succès du film comme une juste récompense pour ses trois ans de collaboration avec Eastwood. Le magazine rajoute ensuite que le nom d'Eastwood est désormais synonyme de succès au box-office.
À cette époque, le fils d'Omar Sharif désire que son père tourne un film de cow-boys. Jack Lee Thompson réalise le film, et Eastwood est prévu en second rôle. Toutefois, le producteur déclare que si le film est une défaite, c'est à cause de lui, Sharif, la principale célébrité du film. Alors, Omar Sharif décide de ne pas prendre Eastwood dans le second rôle, le considérant comme trop peu connu. Il engage Gregory Peck. Ils tournent ensemble L'Or de MacKenna alors qu'Eastwood tourne Pendez-les haut et court. Le premier film ne marche pas du tout, alors que le second fait un carton et permet à Eastwood d'accéder au rang de star. C'est depuis ce temps que Sharif déclare à Eastwood, quand il le croise : « n'oublie pas, c'est moi qui t’ai fait ! ».
Mais United Artists est à l'époque une « petite » société, et les productions ne s'y enchaînent pas comme à Universal. Eastwood n'écarte pas l'idée de retourner un jour chez le major qui l'a jadis rejeté. D'ailleurs, le contrat de Clint avec UA n'est pas un contrat d'exclusivité, et Malpaso peut, quoi qu'il arrive, traiter avec toutes autres sociétés. Ainsi, Jennings Lang permet à l'acteur de signer un nouveau contrat avec Universal, avec un salaire à plus du double de celui d'UA. Ce dernier lui assure le premier rôle dans le prochain long métrage de Don Siegel, Un shérif à New York, qui devait initialement être un feuilleton, avant que le script ne soit réécrit par Dean Riesner. Clint désire incarner un « connard héroïque », selon ses mots, cherchant à se distinguer de ses précédents rôles,.
Clint, col bleu : un nouveau genre

Kyle Eastwood (2005)
Le 19 mai 1968, quelques mois avant la sortie du film Un shérif à New York, Maggie Johnson accouche de leur premier enfant : Kyle Eastwood. Cela faisait plusieurs années que Maggie souhaitait avoir un enfant, sans jamais y parvenir. Si Clint se dit très fier de ce nouveau venu, il ne peut pas rester longtemps à en profiter, face à ses obligations professionnelles. Cette période de sa vie est sans doute la plus active. Il accepte de tourner dans le film Quand les aigles attaquent de Brian G. Hutton malgré un salaire plus bas qu'à l'accoutumée. Ce film lui permet d'affirmer sa position en Europe et de changer de genre,. Par la suite, il accepte un rôle dans La Kermesse de l'Ouest (1969), un nouveau film qui se démarque de tous les autres puisqu'il s'agit d'un film musical. C'est une adaptation d'un spectacle de Broadway sur la ruée vers l'or. Le rôle d'Eastwood a été spécialement créé pour l'acteur, car absent du script original. Mais le tournage ne se déroule pas comme prévu. Si le film se voit attribuer un budget imposant, cela comprend le salaire de toutes les célébrités engagées sur le tournage, celui des choristes et le coût des décors. Qui plus est, une météo peu clémente détruit tous les décors extérieurs et le réalisateur, Joshua Logan, apparaît instable,. Quoi qu'il en soit, l'épouse de Clint Eastwood se plaint de la longue absence de son mari vis-à-vis de leur jeune enfant :
« Je veux que Clint soit plus proche de son fils. Je vous assure qu'il l'a à peine regardé jusqu'à présent. Il enchaîne les films à une telle vitesse qu'il n'est à la maison qu'une ou deux fois par semaine. La dernière fois qu'il est rentré d'un tournage, il faisait nuit. Kyle dormait et Clint a à peine jeté un coup d'œil sur lui. C'est un peu dur pour un jeune père. »
Insérer un paragraphe

Le Morelos où a été tourné Sierra torride
Fin 1969, Leonard Irving, le président de la Malpaso et conseiller financier d'Eastwood, meurt. Cet événement aura un effet néfaste sur l'acteur. Leonard est remplacé chez Malpaso par Bob Daley, un vieil ami de Clint, et Roy Kaufman devient son conseiller financier. Durant cette période, l'acteur se lasse d'Universal qui ne lui propose que des films dont le script est plat. Par ailleurs, le contrat signé entre la Malpaso et Universal stipule qu'en cas de désaccord majeur entre les deux sociétés, Universal a le dernier mot. Eastwood n'aime pas ce sentiment d'emprisonnement. Le premier différend entre les deux sociétés survient avec leur coproduction de 1970 : Sierra torride. Personne ne se met d'accord sur le script. Le contrat a été signé avant le succès de Pendez-les haut et court et Un shérif à New York, et Eastwood ne peut plus revenir en arrière. Lors de la signature, il devait donner la réplique à Elizabeth Taylor, mais c'est Shirley MacLaine qui est finalement retenue. Taylor souhaitait tourner en Espagne, ce qu'approuvaient Eastwood et Siegel, mais pas Universal. Finalement, la critique est moins généreuse avec ce film. Le meilleur commentaire paraît dans le New York Times : « il est néanmoins bon, et il reste dans la mémoire ». Sierra Torride fait tout de même partie du classement des mille meilleurs films de tous les temps, publié dans le The New York Times Guide to the Best 1000 Movies Ever Made.
Ensuite, Eastwood ne dispose que de quelques mois pour profiter de son fils avant de repartir sur le tournage de De l'or pour les braves. Tournage qui ne se déroule pas comme prévu : la météo est une fois de plus peu clémente, et les effets spéciaux ne fonctionnent pas comme voulu, brûlant par exemple les décors. Lorsque le tournage touche à sa fin, Eastwood n'en peut plus. C'est ainsi le dernier contrat qu'il passe hors Malpaso. Il enchaîne avec Les Proies, réalisé par Don Siegel durant l'hiver 1969. Julian Blaustein est nommé producteur par les studios, mais, après avoir critiqué Eastwood, trouvant que « le rôle aurait dû être attribué à un homme plus jeune et d'apparence plus innocente », il est rapidement évincé. Clint Eastwood incarne finalement ce jeune soldat nommé McB qui se réfugie dans un camp de filles, qu'il charme une à une en leur racontant des mensonges. Un critique parisien, Pierre Rissient, propose d'organiser la première du film au Festival de Cannes 1971. Mais Hollywood étant peu à l'aise à Cannes à l'époque, Universal refuse cette offre, bien qu'Eastwood la trouve géniale. Le studio s'occupe seul de la distribution du film, que Schickel considère comme « stupide ». À la fin de sa distribution aux États-Unis, le film ne fait pas de bénéfice, rapportant à peine 1 000 000 $. Mais Rissient est toujours enthousiasmé par Les Proies et s'arrange pour que des critiques français le voient avant sa sortie. Après la première à Paris, Paris Match en publie une critique élogieuse, le trouvant « étrange et violent, comme les nouvelles d'Ambrose Bierce ».
Les débuts comme réalisateur

Clint Eastwood, en 1970, lors d'un tournoi de tennis
En 1970, Eastwood perd son père, Clinton Eastwood, qui meurt d'une crise cardiaque. Il abandonne durant plusieurs semaines son projet suivant : Un frisson dans la nuit. Quand il revient, Eastwood est différent : il ne boit plus d'alcool fort et fait davantage attention à sa santé. Voici les propos de Fritz Manes sur cette époque de sa vie :
« Il a été complètement dévasté par la mort de son père, et ce parce que c'était la seule mauvaise chose qui lui était jamais arrivée dans la vie. Jusqu'ici les problèmes qui survenaient étaient toujours réglés à la fin de la journée. Il n'arrivait pas à comprendre ce qui lui arrivait. Il l'a pris comme une affaire personnelle — comme si on lui avait fait quelque chose, à lui, personnellement. Il a mis beaucoup de temps à s'en remettre, et Bon Dieu, il a bien failli s'effondrer. »
 
Un frisson dans la nuit, le dernier projet qu'a conclu Leonard Irving avant de mourir, est le premier film d'Eastwood en tant que réalisateur ; il y joue également le rôle principal. Ce film lui permet d'aborder le thème du jazz, et d'explorer une nouvelle fois la psychose sexuelle, après Les Proies qui met en scène un soldat qui devient l’objet du désir de plusieurs femmes, ce qui lui permettra de survivre. La mort de son père a eu un impact sur le style visuel du film, très sombre et mélancolique. Le tournage se déroule mieux que prévu : il coûte 50 000 $ de moins et se termine quatre jours avant la date fixée. Rissient, qui est devenu représentant européen de l'acteur, organise une projection du film et la première rétrospective de l'œuvre d'Eastwood au Festival du film de San Francisco en 1971,. Mais cette première apparition dans un festival n'est pas une réussite : Eastwood n'est en particulier pas très apprécié des féministes. Mais cela n'empêche pas le film d'être accueilli chaleureusement lors de sa sortie en salles. Quoi qu'il en soit, Un frisson dans la nuit constitue un nouveau tournant dans la carrière de l'acteur : en plus d'être pour la première fois réalisateur, Eastwood ne joue plus un cow-boy ni un militaire.
La même année, Current Biography estime les recettes totales des films d'Eastwood sur le marché mondial à environ 200 000 000 $, tandis que Life consacre Eastwood « star du cinéma la plus populaire du monde ». À cette époque, Eastwood n'a plus de projet de films, jusqu'à ce que Jenning Langs lui présente le script de L'Inspecteur Harry. C'est l'histoire de Harry Calahan, un policier de San Francisco déterminé à arrêter un meurtrier psychotique par tous les moyens. Le script, qui appartenait initialement à Universal, est finalement acheté par la Warner Bros. Frank Sinatra est un temps pressenti pour interpréter le rôle principal, mais le projet traînant en longueur, l'acteur abandonne finalement pour laisser sa place à Eastwood. Harry est un solitaire qui vient de perdre sa femme et qui ne cherche plus l'amour. Andrew Robinson est choisi par Siegel pour interpréter le méchant du film. C'est ainsi que commence le tournage de L'Inspecteur Harry.
Harry le charognard

Les studios Old Tucson où débute le tournage de Joe Kidd
Le tournage s'avère difficile au début. Il comprend de nombreuses cascades périlleuses, complexes à mettre en place. Par ailleurs, Siegel attrape la grippe et doit s'absenter plusieurs jours du tournage : Eastwood décide de prendre sa place pour diriger le film lui-même durant son absence. L'Inspecteur Harry est clairement opposé à l'avertissement Miranda qui vise à informer un suspect de ses droits constitutionnels avant un interrogatoire,. D'ailleurs The New Yorker publie une critique franche en janvier 1972 : « ce genre de film d'action a toujours recélé un potentiel fasciste, qui a fini par faire surface. […] Nous aurions tous la permission de tuer, comme Dirty Harry (...Je pue peut-être, mais j'ai un gros flingue !...) Mais dans la mesure où le crime est causé par la dépravation, la misère, la psychopathologie, et l'injustice sociale, L'Inspecteur Harry est un film profondément immoral ».
Harry Callahan symbolise pour l'Amérique un nouvel essor, un renouveau : à l'époque, alors que la police est contestée et que la guerre du Viêt Nam risque d'être une défaite, il incarne le héros dont les Américains ont besoin. En décembre 1971, le film sort en salle (début 1972 en France). Il se hisse très rapidement en tête du box-office, pour finalement rapporter en fin d'exploitation plus de 53 000 000 $. Grâce à ce film, Eastwood devient la célébrité d'Hollywood la plus lucrative. Malgré quelques mauvaises critiques concernant le côté « fasciste » du film, il a été généralement très bien accueilli. Au moment de la sortie de L'Inspecteur Harry, Richard Nixon annonce au peuple américain qu'il compte se présenter une nouvelle fois aux élections présidentielles. Eastwood déclare qu'il soutient Nixon, et est invité à plusieurs repas officiels. C'est ainsi qu'il se voit offrir un mandat de six ans au National Council of the Arts, un organisme consultatif sur les subventions fédérales à apporter aux initiatives artistiques, bien que le nom de Cesar Romero ait d'abord été avancé.
À son nouveau poste, Eastwood privilégie les petits artistes américains, notamment ceux du milieu du jazz. Néanmoins, selon les archives du conseil, il ne participe pas souvent aux réunions organisées : il n'assiste qu'à cinq des vingt-cinq réunions entre 1972 et 1976. C'est en raison de ce manque d'assiduité qu'on demande à Eastwood de démissionner avant le terme de son mandat.
Son film suivant, Joe Kidd (1972), relève du même style que les précédents : un homme solitaire tente de faire la loi. L'histoire originelle s'inspire de la vie d'un leader passionné de la lutte pour la terre, Reies Lopez Tijerina. Mais Elmore Leonard l'a sensiblement modifiée pour que le personnage d'Eastwood devienne le héros. Durant le tournage, Clint attrape une grippe qui se transforme ensuite en allergie sous forme de bronchite. Cela provoque le retour de ses crises d'angoisse, mais elles s'estompent lorsqu'il va voir Roxanne Tunis, qui lui enseigne la méditation transcendantale. Il déclare à ce sujet lors d'une interview en 2009 :

L'Homme des Hautes Plaines
Insérer un paragraphe
« Méditez-vous encore ?
— Deux fois par jour.
— Comment cela fonctionne-t-il pour vous ?
— Cela fonctionne très bien, parce que cela vous donne l'occasion de rassembler vos idées, je suis très scrupuleux à ce sujet quand je travaille. »
Clint Eastwood décide de passer pour la deuxième fois derrière la caméra avec L'Homme des Hautes Plaines. C'est le premier western qu'il tourne lui-même. Ce film est très proche, par l'intrigue et la mise en scène, de ceux tournés avec Sergio Leone : un étranger arrive dans une ville et il est engagé pour la protéger de trois méchants. Le tournage s'effectue dans les environs du lac Mono, où Henry Bumstead crée un décor mystérieux. Et, dans la scène finale, Eastwood fait un clin d'œil à trois de ses principales collaborations : alors qu'il part à cheval, on voit, gravés sur des pierres tombales, les noms de Don Siegel, Sergio Leone et de Brian G. Hutton. Eastwood déclare à la presse qu'il « enterre [ses] réalisateurs ». Cette même année 1972, sa femme donne naissance à leur deuxième enfant : Alison Eastwood. Maggie Johnson est d'autant plus heureuse qu'elle souhaitait depuis longtemps avoir une fille. Eastwood fait figure de mari modèle depuis bien des années, mais ses conquêtes ne passent plus inaperçues aux yeux de la presse. On peut désormais lire dans les premières pages des journaux des titres comme « Pourquoi dit-on que Clint Eastwood est le pire mari de tout Hollywood ? »
L'âge adulte : Warner Bros.
Une suite à Harry ?
Juste avant la sortie en salles de son dernier film, Eastwood contacte Elmore Leonard pour lui demander s'il a un nouveau script en tête, quelque chose de similaire à l'Inspecteur Harry. En fait, si Clint Eastwood veut tourner une suite à Harry, c'est à cause du trop faible pourcentage qu'il touche sur les recettes du film. Leonard lui propose un script qu'il a ébauché : un cultivateur d'artichauts qui vit à Castroville refuse de céder face à une association de malfaiteurs qui veut lui extorquer de l'argent. Eastwood refuse ce projet, surtout parce que Castroville est bien trop proche de Carmel où il vit. Leonard finalise tout de même son projet en le modifiant. Mr. Majestyk est tourné en 1973 avec Charles Bronson à la place d'Eastwood.

Clint Eastwood et William Holden sur le tournage de Breezy en 1973
Elmore Leonard parle toutefois d'une suite de L'Inspecteur Harry à Clint qui met tout en œuvre pour voir le projet éclore. Mais, entre-temps, Jo Heims lui propose un tout autre rôle : celui d'un agent immobilier propulsé dans une relation avec une fille « libérée ». Eastwood déclare ainsi à la presse : « J'ai bien cerné le personnage. Elle voulait que je le joue. Je lui ai dit : « Jo, je ne crois pas que je sois dans la bonne tranche d'âge ». Il accepte néanmoins de le mettre en scène et Breezy est tourné en 1973. Le film n'est pas une grande réussite : il n'a ni marqué le public, ni la critique.
Alors que Breezy ne marche pas, la Warner annonce qu'Eastwood va reprendre le costume de L'Inspecteur Harry pour une suite, d'abord nommée Vigilance. Au début, les scénaristes essaient de travailler dans la continuité du premier film, mais ils changent vite d'avis. Dans Magnum Force, son titre définitif, Harry gagne en charme, et il a désormais une relation avec une femme. Ce deuxième film comporte une intrigue plus simpliste. Durant le tournage, Eastwood, également producteur du film, a souvent été opposé au réalisateur, Ted Post, car il veut économiser le budget. Lors de sa sortie en salle, le film est dénigré par les critiques. On peut lire « toujours le même truc » ou encore « moralisme embrouillé » dans des périodiques américains. La plus dure est écrite par Pauline Kael : « comme [Clint Eastwood] n'est pas acteur, on peut difficilement le traiter de mauvais acteur. Il faudrait qu'il fasse quelque chose pour que l'on puisse évaluer ses qualités de comédien. » Malgré tout, le film réalise un meilleur score au box-office que L'inspecteur Harry, rassemblant en fin d'exploitation plus de 39 000 000 $.

L'Eiger, où a lieu le tournage de La Sanction
Stan Kamen propose ensuite à Eastwood le script du Canardeur. C'est l'histoire d'un braqueur de banques, vétéran de la guerre de Corée, qui cherche à garder une longueur d'avance sur les membres de son groupe. Il se lie d'amitié avec un autre homme, surnommé « Pied de biche », durant un voyage. Ensemble, ils vont récupérer l'argent qu'a déjà caché le braqueur avant que quelqu'un ne s'en empare. Kamen refuse de vendre le projet si son auteur, Michael Cimino, n'en est pas le réalisateur. Eastwood accepte de rencontrer Cimino, qui n'est autre que le scénariste de Magnum Force, et ils s'entendent sur le projet. La Warner refuse au dernier moment de le produire, le trouvant trop atypique pour Eastwood, mais la United Artists reprend l'affaire. Clint n'apprécie pas beaucoup ce film parce que Jeff Bridges lui vole la vedette ; d'ailleurs, Bridges est nommé pour un Oscar et pas Eastwood. D'autre part, le film, dont beaucoup louent les qualités, ne remporte pas le succès attendu, ce qui l'énerve passablement. C'est pourquoi le deuxième film prévu entre Malpaso et UA ne voit finalement pas le jour.
Eastwood tourne ensuite, en tant que réalisateur et acteur, La Sanction, une adaptation du roman de Trevanian The Eiger Sanction. C'est l'histoire d'un universitaire spécialiste en histoire de l'art à qui l'on demande de reprendre du service en tant que tueur à gages, son ancien métier, pour exécuter une dernière mission en échange du tableau d'un grand artiste. Le film est produit par Richard Zanuck et David Brown. Le tournage a lieu dans les Alpes suisses, pour donner un côté réaliste à la prise de vues. Le tournage manque d'être arrêté à cause des conditions extrêmes, et l'impatience d'Eastwood met en danger plusieurs techniciens,. Lors de sa sortie, le film est boudé par la critique. On peut même lire que c'est une « farce grotesque ». Cela se répercute sur la réception du public qui s'avère très mauvaise. En fin d'exploitation, le film rapporte seulement 14 000 000 $.
Prémices d'une longue collaboration

Paria, ville fantôme où est tourné Josey Wales hors-la-loi
Frank Wells, le vice-président de la Warner Brothers, apprécie beaucoup Eastwood. Sa société détient par ailleurs tous les droits sur la saga de L'Inspecteur Harry. Clint Eastwood, à cette époque, est déçu de son film précédent, qui n'a pas marché, alors qu'il misait beaucoup dessus. Wells le persuade alors de signer avec la Warner, en septembre 1975, un contrat très avantageux. Le studio s'engage à tirer un grand nombre de copies des films d'Eastwood, accompagnées de campagnes publicitaires sans précédent. De plus, Eastwood peut enfin se prononcer sur la stratégie publicitaire à adopter, et sa société, Malpaso, gagne en importance,.
Leur première collaboration, initiée par Malpaso, est Josey Wales hors-la-loi. Eastwood engage Philip Kaufman pour l'écriture du scénario et la réalisation. Mais au bout de quelques jours de tournage, il le renvoie,. Les deux hommes ont des avis très divergents, et la méthode de prise de vues de Kaufman ne convient pas à l'acteur. Eastwood prend dès lors les commandes de la réalisation. Cette affaire fait beaucoup de bruit en Amérique, et mécontente la Directors Guild of America qui édicte une règle, baptisée la « règle Eastwood » qui punit ce genre d'action. Finalement, à sa sortie, le film est apprécié par la critique. Il est aujourd'hui considéré comme « l'une de ses œuvres les plus profondes, les plus personnelles ». Cette bonne réception est en partie due à l'ampleur de la campagne publicitaire et aux invitations luxueuses offertes aux critiques.
« Eastwood est un interprète si taciturne et intéressé par l'action qu'il est facile d'oublier le fait qu'il réalise beaucoup de ses films — et beaucoup parmi les meilleurs et les plus intelligents. Ici, avec l'humeur changeante, la beauté humble et la photographie de Bruce Surtees, il a réussi à créer un western sentimental magnifique. »
— Roger Ebert, au sujet de Josey Wales hors-la-loi, Chicago Sun-Times
Gail Morgan Hickman et S. W. Schurt, deux anciens élèves de l'école secondaire d'Oakland et grands admirateurs d'Eastwood, lui proposent un script de cent dix pages de leur invention. La Warner considère que l'intrigue a un bon potentiel, mais que le scénario nécessite une réécriture. Alors, Eastwood renvoie le texte aux deux jeunes, qui passent beaucoup de temps à retravailler leur histoire, pour finalement rendre un travail plus mauvais. Avec cette perte de temps, Eastwood perd son statut de numéro un au box-office, et en même temps six mois de production qui lui coûtent cher. Le script devient la nouvelle suite des aventures de Harry Callahan, cette fois opposé à un groupe de terroristes. Eastwood engage Stirling Silliphant, qui vient de terminer une collaboration avec Don Siegel, pour adapter le scénario. La proposition majeure qu'il fait est d'associer Callahan avec une femme, et son histoire s'intitule L'inspecteur ne renonce jamais, titre du nouvel épisode de la saga. Par ailleurs, si Eastwood est pressenti pour la réalisation, c'est James Fargo, à la surprise générale, qui occupe finalement le poste. En fin de compte, le film est un grand succès, avec une recette de 46 236 000 $ aux États-Unis. Le côté féministe d'Eastwood est très apprécié, même s'il est élu « pire acteur de l'année » par Harvard Lampoon. Il s'agit du plus grand succès, à l'époque, de l'acteur.
Les années Locke
Cela fait plusieurs mois, depuis Josey Wales hors-la-loi, qu'Eastwood a une relation avec l'actrice Sondra Locke. En parallèle, il se penche sur son prochain film : L'Épreuve de force, pour lequel il cherche une actrice devant interpréter un personnage lié à la mafia. La Warner pense d'abord à Barbra Streisand, une valeur sûre compte tenu du coût du scénario. Dennis Shryack et Michael Butler ont en effet vendu leur premier script à Universal pour 300 000 $ et 15 % des bénéfices, mais pour L'Épreuve de force, ils demandent 500 000 $ et 15 % des bénéfices. Toutefois, Eastwood voit mieux Sondra Locke dans le rôle, ce que les deux scénaristes approuvent quand ils découvrent un aspect fragile mêlé au côté dur de sa personnalité. Avec ce film, Eastwood reprend le poste de réalisateur. Et, comme à son habitude, le tournage s'effectue très rapidement, Eastwood préférant la spontanéité de la première prise. Plusieurs critiques lui reprochent d'avoir trop mis l'accent sur les scènes de violence. Selon d'autres, comme Arthur Kinght dans le Hollywood Reporter, c'est le jeu de Sondra Locke qui relève le film. Le film se révèle néanmoins être l'un des dix plus grands succès de l'année 1977.

Lieu de tournage de Doux, dur et dingue (Albuquerque)
La fin du tournage de L'Épreuve de force marque le déménagement de Sondra Locke à Sherman Oaks (Los Angeles) pour se rapprocher d'Eastwood. Ils font ensemble la couverture du magazine People, pour leur deuxième collaboration. Eastwood y porte un regard ambigu sur Locke, et il la surnomme « princesse ». C'est ainsi que Maggie Johnson, l'épouse d'Eastwood, apprend sa relation avec l'actrice. Elle appelle donc un avocat. La famille Eastwood effectue un dernier voyage à Hawaï, durant lequel Maggie Johnson espère sauver son couple. À son retour, l'acteur annonce à Locke que sa femme compte demander une séparation légale, et non un divorce. Cela faisait déjà une dizaine d'années que le couple allait mal, mais Maggie Johnson espérait se tromper quant à son mari. Un jour, Locke convainc son compagnon de tourner une comédie, Doux, dur et dingue, ce qui marque un brusque changement de cap dans la carrière de l'acteur et réalisateur. Les producteurs de Malpaso et de Warner sont d'ailleurs dubitatifs quant à cette idée. Ils organisent même un sondage auprès du public pour voir s'il assisterait à la projection d'un tel film avec Eastwood. Le résultat est mitigé, mais les studios acceptent finalement de financer le projet. Locke fait à nouveau partie de la distribution. La promotion est assurée par la sortie, peu avant la distribution du film, de la bande originale qui figure parmi les meilleures ventes country de l'année. Pour parfaire la publicité, la Warner diffuse des bandes-annonces à la télévision nationale américaine. Le film sort finalement fin 1978 dans 1 246 salles en même temps. La critique n'apprécie pas ce nouveau film, « tellement mauvais » selon Variety. Mais côté public, il obtient un véritable succès. La Warner, et Eastwood, battent leur record au box-office en enregistrant une recette totale de 85 196 485 $, rien qu'en Amérique.

Prison d'Alcatraz
Pendant la distribution de Doux, dur et dingue, Johnson et Eastwood débattent du partage des biens de l'acteur. Dans un premier temps, il doit trouver une nouvelle maison, laissant l'ancienne à sa femme. Il en achète une à Shasta en Californie, qu'il partage avec sa compagne, Locke. Malgré ce rapprochement, le film suivant d'Eastwood ne comporte pas de rôle pour elle. L'Évadé d'Alcatraz possède une distribution composée quasi exclusivement d'hommes, qui jouent les détenus de la fameuse prison d'Alcatraz. L'histoire se fonde sur l'évasion de trois prisonniers qui a eu lieu en 1962. Eastwood a déjà tourné dans cette ancienne prison pour L'Inspecteur ne renonce jamais. Il accepte le rôle à condition que Malpaso en soit la compagnie productrice. Don Siegel, le réalisateur, qui désire également produire le film, double le salaire de l'acteur et achète le script pour être pleinement propriétaire du projet. Les deux hommes se fâchent quelque peu parce que Siegel décide de présenter le film à Paramount Pictures plutôt qu'à Warner, le studio qui produisait Eastwood jusque là. Dans un premier temps, Siegel se met en quête d'un autre acteur pour le rôle principal, bien que Paramount ait réellement envie de voir Eastwood jouer dans son film, le débauchant ainsi de Warner. Mais Siegel ravale sa fierté et prend rendez-vous avec Eastwood. À la fin de l'entrevue, les deux hommes sont à nouveau amis, et se lancent dans leur cinquième collaboration. Le tournage s'effectue rapidement, au milieu des visites touristiques du pénitencier. La Paramount a dépensé beaucoup d'argent pour la restauration des cellules.
 Malpaso pour la distribution de L'Évadé d'Alcatraz
Ce film est le premier de la collaboration d'Eastwood et de Siegel dont le montage s'effectue sans ce dernier. Eastwood assiste donc à toute la post-production. C'est le plus long des films dans lesquels Eastwood est dirigé par Siegel depuis leurs débuts : il dure 120 minutes. La conclusion contraste également avec beaucoup des films d'Eastwood puisque peu de personnes sont tuées. À sa sortie, en décembre 1979, le film est très bien accueilli par la critique : on parle de « grâce et sérénité cinématographique » et de « cinéma cristallin », et Siegel est catalogué « conteur classique ». Ce film est toutefois le dernier qui réunit au générique les noms de Siegel et Eastwood. Siegel paraît dégoûté de ce qu'Eastwood est devenu. Cela n'empêche pas L'Évadé d'Alcatraz d'être un succès au box-office, avec une recette finale de 43 000 000 $ aux États-Unis ; succès faible cependant au regard des précédents films d'Eastwood.
Une mauvaise passe

Clint Eastwood en 1981
À la fin des années 1970, Sondra Locke doit avorter deux fois d'Eastwood à sa demande. Un jour, elle découvre le script de Bronco Billy par le biais de Malpaso, script qu'elle apprécie particulièrement pour son côté chaleureux, et le propose à Eastwood. « C'était mon projet chouchou. Je l'ai supplié de le faire. Je trouvais ça tellement bien. Je dirais que ça a été le truc le plus personnel de tout ce que j'ai fait avec Clint. » Pour se racheter auprès d'elle, il accepte le projet. Il prend les postes d'acteur et réalisateur et commence la production de Bronco Billy début 1980. Le tournage s'effectue rapidement, en six semaines, avec un budget minimaliste pour l'époque : 5 000 000 $. C'est David Worth, le directeur de la photographie, qui s'occupe de l'éclairage ainsi que de trouver les décors. Il arrive sur les lieux bien avant Eastwood pour tout préparer, afin que les prises de vues puissent débuter dès l'arrivée du réalisateur.
La Warner ne change pas sa stratégie et déploie une vaste campagne publicitaire dont le budget s'élève à 2 650 000 $, avec l'organisation d'avant-premières et une sortie dans 1 316 salles. Tout le monde pense que le film va se placer en tête du box-office dès sa sortie. Mais à la surprise générale, Bronco Billy est la première collaboration entre Warner et Eastwood à faire une mauvaise première semaine : « Le lancement du film n'avait rien de comparable aux hauts standards auxquels Eastwood les avait habitués ». Alan Friedberg, président de la National Association of Theaters Owners, déclare même que « les gens aiment voir Clint Eastwood avec un cigarillo dans la bouche et un flingue dans la main […]. Mais Bronco Billy est une comédie, et il n'y a même pas de chimpanzé ». Tandis que le film rate sa sortie, Eastwood est déjà sur le tournage du film suivant : Ça va cogner, une suite à Doux, dur et dingue. Lorsqu'il apprend la nouvelle, il entre dans une colère terrible, et menace de quitter Warner Brothers. Il décide de prendre désormais en main la campagne publicitaire de ses films. Il crée de nouvelles affiches pour Bronco Billy et des critiques décident de retourner le voir. Beaucoup d'entre eux publient de nouveaux commentaires sur le film ; on lit par exemple « le film ne marche pas au box-office et c'est vraiment dommage », dans le Los Angeles Weekly, ou encore « vraiment très drôle » dans Variety. Malgré tout, le film est une déception pour Eastwood et pour Warner, et l'acteur prend la décision de ne plus tourner de comédie, du moins pour le moment.

Un MiG-25, avion sur lequel se base l'avion fictif MiG-31 Firefox vu dans Firefox, l'arme absolue
L'histoire de Ça va cogner est bien différente de Doux, dur et dingue. Le personnage de Sondra Locke est méconnaissable. L'actrice déclare même à son compagnon : « mais qui suis-je ? Quels sont les liens entre mon personnage et celui d'avant ? », ce à quoi Eastwood répond, irrité, « s'ils remarquent ça… ». Le film est projeté dans 2 560 salles à sa sortie, un nombre record. Sorti à la fin de l'année 1980, il fait partie des films moyens, selon Warner. Et Ça va cogner est à nouveau une déception. Aux États-Unis, il ne réalise qu'une recette de 10 000 000 $ lors de sa semaine d'ouverture.
Eastwood enchaîne en 1981 avec Firefox, l'arme absolue, film qu'il réalise et produit, et dans lequel il interprète le rôle principal. Ce film est la toute première production d'Eastwood. S'il endosse ce nouveau poste, c'est pour passer outre une grève prochaine de la Directors Guild of America qui interdit la présence du réalisateur en salle de montage. Il peut de cette manière assister au montage, en tant que producteur. En parallèle, le Museum of Modern Art de New York organise une journée d'hommage à Eastwood, ainsi qu'une projection de quatre de ses films. À cette époque, l'acteur est las de jouer les durs, il veut en terminer avec Harry le charognard. Et, alors qu'il revoit l'un des scénaristes de L'Épreuve de force, Dennis Shryack, ce dernier lui présente un nouveau script dont le personnage central est un flic solitaire, d'âge mûr, qui lutte contre le crime organisé, mais qui perd toutes ses batailles. Eastwood y voit l'occasion idéale de donner une fin à L'Inspecteur Harry, ce serait sa dernière mission. Mais le décès de Jo Heims, scénariste de deux de ses films et du même âge que lui le bouleverse beaucoup. Peu après, Eastwood reçoit le script de Shryack, mais il déteste l'histoire, alors que c'est lui-même qui l'a demandée. Acheté par Andrew Davis, ce script devient finalement Sale temps pour un flic, et c'est Chuck Norris qui interprète le rôle principal. Peut-être pour se racheter, Eastwood demande à Shryack de lui écrire un western classique, dans le genre de L'Homme des vallées perdues, et publie une annonce dans laquelle il déclare être à la recherche d'un nouvel Inspecteur Harry.
L'homme politique, père dévoué et féministe
Eastwood a débuté sous le mandat d'Eisenhower, il s'est fait reconnaître du public durant celui de Nixon, et contribue ensuite à la célébrité de Reagan. En 1980 et 1984, l'acteur soutient la campagne de ce dernier, dont le slogan n'est autre qu'une réplique du Retour de l'inspecteur Harry : « Vas-y, fais-moi plaisir ! ». Dans son élan patriotique, Eastwood finance le projet de James Gritz : une expédition menée à deux reprises pour libérer des prisonniers au Laos. Ces expéditions sont des échecs puisque aucun prisonnier n'est découvert, et qu'en outre deux mercenaires américains y perdent la vie. Eastwood, en homme taciturne, ne daigne pas parler publiquement de cette affaire. Mais il envisage de l'utiliser pour son prochain Inspecteur Harry. Par ailleurs, si l'on met de côté Ça va cogner, la première production de Malpaso durant cette décennie Reagan, Firefox, l'arme absolue, démontre un retour manifeste à la Guerre froide comme source d'inspiration.

Une Lincoln Continental (Honkytonk Man)
En attendant de trouver un bon scénario pour les prochaines aventures de Harry Callahan, Eastwood décide d'adapter un roman de Clancy Carlile, Honkytonk Man, une tragédie émouvante et pittoresque sur les rêves puis la mort d'un chanteur de musique country. Les droits appartiennent à la William Morris Agency de New York, qui souhaite que le film soit adapté par un de ses clients. Dans le livre, l'histoire est racontée du point de vue du neveu du musicien, âgé de 14 ans. Or Eastwood a déjà confié aux journalistes qu'il souhaite que son fils suive sa propre trace. Il voit dans ce film l'occasion parfaite pour le lancer. Carlile n'est pas un grand admirateur d'Eastwood et pense que l'acteur ne correspond pas à son personnage, mais Clint est le principal client de la William Morris Agency, et il a, de plus, une certaine expérience de la musique. Eastwood exige aussi d'être la star du film. Finalement, Carlile accepte, après avoir rencontré Eastwood dans son ranch. Il s'occupe par ailleurs lui-même d'écrire le scénario du film, bien qu'Eastwood souhaite modifier légèrement l'histoire originale.
Le film est tourné durant l'été 1982. Carlile est déçu par l'interprétation d'Eastwood, considérant qu'« il a échoué lamentablement ». On voit un personnage sexy et frais à la place d'un homme rongé par la tuberculose et par l'alcool. À la sortie d'Honkytonk Man les critiques sont généralement mauvaises : le fond est intéressant, mais l'ensemble est ennuyeux. Ces critiques font toutefois l'éloge de Kyle Eastwood, et Clint est ainsi reconnu comme un père dévoué et réfléchi.
En mai 1984, le divorce entre Maggie Johnson et Clint Eastwood est enfin prononcé. Elle touche une grosse somme d'argent, obtient la garde des enfants et leur maison de Pebble Beach. L'acteur, de son côté, a le droit de visite libre de ses enfants. Pendant ce temps, Sondra Locke découvre un script susceptible d'intéresser Eastwood. Elle a joué, hors Malpaso, dans un film écrit par Earl E. Smith et Charles B. Pierce, The Shadow of Chikara. Les deux hommes lui ont aussi promis un scénario où elle aurait le rôle principal, sans Eastwood,. Lorsqu'elle l'obtient, elle le montre à Eastwood pour savoir ce qu'il en pense. Ce dernier adore le script et le trouve parfait pour lui-même. Il y voit le prochain Inspecteur Harry, après quelques retouches, et promet à Locke le premier rôle féminin. Dean Riesner est engagé pour réécrire le scénario, mais il est finalement renvoyé pour laisser place à un inconnu, Joseph Stinson. Il coûte moins cher à Eastwood et Fritz Manes, le producteur, triple le salaire de Locke. Commence ainsi en 1983 le tournage du quatrième Harry, Le Retour de l'inspecteur Harry. Si le film n'est qu'un pâle reflet des précédents, Harry Callahan est toujours populaire, et le succès est important, supérieur à celui de tous les épisodes précédents. Pour une fois les critiques ne sont pas acerbes, le film étant sous-tendu par des considérations morales, et qualifient Eastwood de féministe.
Clint Eastwood : un artiste

Richard Benjamin tourne Haut les flingues ! au gré de l'humeur d'Eastwood
À la même époque, Richard Tuggle écrit un nouveau script, s'inspirant d'articles de journaux qui traitent d'un violeur toujours en liberté. Il présente son histoire à Don Siegel dans l'espoir que ce dernier le produira et lui permettra de le réaliser avec Eastwood dans le premier rôle. Eastwood adore l'histoire et accepte directement la proposition, mais Siegel la refuse. Eastwood incarne un détective d'âge mûr, qui vient de divorcer et prend à cœur l'éducation de ses deux enfants. Toutefois, il trouve un certain réconfort, la nuit, dans l'alcool et le sexe. Si Kyle Eastwood a eu l'opportunité de jouer dans Honkytonk Man, c'est Alison, sa fille, qui obtient cette fois-ci un rôle. Geneviève Bujold obtient le premier rôle féminin. C'est Tuggle qui réalise le film, qui sort en août 1984 sous le titre La Corde raide. Mais les critiques regrettent le côté sexuel trop explicite ; le Los Angeles Times trouve les fantasmes « hautement insipides », le film est comparé à une « vente de corps de femmes au box-office ». Il rapporte tout de même près de 50 000 000 $ aux États-Unis. Alors, en voyant ce succès commercial, certains critiques revoient leur jugement, et publient finalement une opinion plus favorable. On peut désormais lire, par exemple dans Village Voice, que c'est « le film le plus fin, le plus réflexif et le plus réfléchi qu'ait fait Eastwood ».
Eastwood en est déjà à son film suivant, Kansas City Jazz, que doit réaliser Blake Edwards. Au départ, il n'est pas intéressé par le projet, mais Edwards organise une réunion, à l'aide de Sondra Locke à laquelle il promet un rôle dans le film. À la fin de la réunion, Eastwood accepte d'y jouer, et Edwards supprime alors Locke de la distribution. Clint Eastwood ne fait rien pour que sa compagne réapparaisse dans le casting. En effet, Le Retour de l'inspecteur Harry marque la dernière collaboration du couple. Toutefois, le film n'entre pas en tournage, il est même annulé pour « différends créatifs » en février 1984, car Edwards et Eastwood n'arrivent pas à se mettre d'accord, et la pré-production se déroule dans une atmosphère de lutte de pouvoir. Quoi qu'il en soit, Fritz Mane devient le nouveau producteur, Joseph Stinson le scénariste et Malpaso la société productrice. Le titre est changé en Haut les flingues !. Le tournage débute avec Richard Benjamin à la réalisation. Celui-ci ne tient pas tête à Eastwood, et lorsque l'acteur n'approuve pas la réalisation, il s'empresse de tout modifier pour convenir à la star. Le tournage est aussi troublé par plusieurs incidents, comme le remplacement d'une actrice française, tombée malade, par Madeline Kahn. Finalement, grâce à la notoriété d'Eastwood et de Burt Reynolds, le film rapporte en fin d'exploitation près de 40 000 000 $. Mais il reste une déception pour la Warner, vu le salaire demandé par les deux acteurs.

La Cinémathèque française organise une rétrospective de vingt-quatre des films d'Eastwood
La star bénéficie, cependant, d'un élan d'enthousiasme aux États-Unis et à l'étranger, grâce aux critiques qui le qualifient d'« artiste provincial le plus important d'Amérique » ou encore de « star de l'économie de l'offre » ; ses films, quant à eux, sont une « part importante de la culture américaine ». Le 24 février 1985, un Eastwood vêtu d'un costume très élégant fait la couverture d'un numéro du New York Times où l'on peut lire « Clint Eastwood, Seriously ». Le chef du bureau parisien du New York Times le suit même lorsqu'il arrive à la Cinémathèque française pour assister à une rétrospective de vingt-quatre de ses films. C'est à cette occasion que l'acteur est promu « Chevalier des Arts et des Lettres » par le ministère de la Culture, bien que le ministre responsable, Jack Lang, soit absent lors de la cérémonie. L'acteur se dirige ensuite vers Munich, en RFA à l'époque, où le Filmmuseum organise également une rétrospective.
Grâce à cet enthousiasme autour d'Eastwood, les critiques qui ne l'apprécient pas se retrouvent vite isolés, ou changent d'avis vis-à-vis de l'acteur et réalisateur. Le mauvais souvenir laissé par Haut les flingues disparaît vite avec le tournage de Pale Rider, le cavalier solitaire, un western artistique dans lequel Eastwood incarne l'étranger typique, similaire à ceux qui défendaient les pionniers dans les années 1950. Ce film est marqué par l'habituelle présence aux côtés d'Eastwood de Lennie Niehaus, Joel Cox, Edward C. Carfagno et Bruce Surtees. Pour ce dernier, directeur de la photographie, il s'agit de sa dernière collaboration. Eastwood prend le pari de tourner le film à la lumière naturelle. Toutefois, comme Shryak le remarque sans pour autant le confier à Eastwood, le film paraît trop sombre, et il doit plisser les yeux pour discerner les éléments. Aussi, lors de la diffusion du film à la télévision, les chaînes décident toutes d'augmenter la luminosité. Comme d'habitude, le tournage est bouclé dans les temps. Toutefois, si les critiques tels que Duane Byrge du Hollywood Reporter acclament la capacité d'Eastwood à terminer ses tournages à la date prévue, la réalité est qu'il bâcle certaines scènes pour y parvenir. En effet, dans Pale Rider, la conclusion du film est traitée de manière expéditive, tout comme les transitions finales. Manes déclare à ce sujet : « Tout à coup, tout se précipite. Les détails qui étaient là au départ, ou au milieu, ont disparu. » Malgré tout, le film entre en sélection officielle au Festival de Cannes. Un reporter du Los Angeles Times rapporte que certains critiques trouvent l'œuvre trop commerciale pour faire partie de la sélection. Le film ne remporte finalement aucun prix. Il est néanmoins un réel succès critique et devient le western le plus lucratif de la Malpaso, à l'époque. L'année 1985 reste finalement, dans la carrière d'Eastwood, celle où il a réussi à se faire reconnaître en tant qu'artiste.
Maire de Carmel

Le Hog's Breath Inn dont Eastwood est copropriétaire
Depuis que Ronald Reagan a été élu président, les journalistes ne cessent de demander à Eastwood s'il envisage de s'investir en politique. Rétorquant que cela ne les regarde pas, l'acteur se présente toutefois en 1986 au poste de maire de Carmel. Dans cette ville se trouvent une de ses résidences principales, des locaux destinés au montage de ses films ainsi que le Hog's Breath Inn, un bar dont il est copropriétaire. Pour les besoins de sa campagne — Clint Eastwood a horreur de l'échec — il engage une conseillère électorale, Eileen Padberg, qui a déjà travaillé pour Reagan. Le soir de l'élection, plus de deux mille journalistes du monde entier sont à Carmel pour rapporter les résultats. Le 8 avril, finalement, Eastwood est élu maire avec 72 % des voix,. À cette époque, il désire à nouveau tourner un film de guerre. Deux ans auparavant, la Warner a reçu un script écrit par un vétéran du Viêt Nam. Eastwood l'invite à venir aux bureaux de la Malpaso. L'entretien entre les deux hommes débouche sur Le Maître de guerre. Si Clint désire conserver l'approbation des critiques en tant que réalisateur, il souhaite également gagner l'image d'acteur de talent. Aussi se met-il volontairement la pression pour interpréter le rôle de Tom Highway dans ce film d'archives. Il déclare à l'époque qu'il n'a jamais eu de rôle aussi profond. Pourtant, malgré le peaufinage du scénario par Dennis Hackin, Joseph Stinson puis Megan Rose, l'analyste-scénario d'Eastwood, le travail rendu est approximatif. Fritz Manes déclare même : « ce film est une véritable perte de temps ». Eastwood le présente tout de même à l'US Army pour obtenir son approbation, ce qui lui permettrait de tourner sur de vrais terrains militaires. Mais elle refuse, estimant que le personnage principal n'est qu'un stéréotype désuet. Manes a alors l'idée de retranscrire l'histoire du point de vue d'un soldat des Marine Corps. Ces derniers sont plus libéraux que l'armée de terre, plutôt conservatrice. Finalement, après un dernier effort sur le scénario, le lieutenant-colonel John Peck accepte. Le tournage commence à la fin de l'été 1986, tandis que tout est mis en œuvre pour que l'acteur ne manque pas les conseils municipaux. Tout se passe pour le mieux, jusqu'à ce que le département de la Défense bloque le projet. Son secrétaire, Bob Simms, n'apprécie pas la vulgarité qui ressort du film. Eastwood retravaille à nouveau le script pour qu'il convienne à tout le monde, et le tournage reprend enfin. Lors de sa distribution, le film est projeté dans 1 470 salles aux États-Unis et le public l'accueille chaleureusement. Optimiste, la Warner organise une campagne dans le but de voir le film sélectionné aux Oscars, mais en vain.

Jim Carrey (ici en 2008) réalise ses débuts au cinéma grâce à Eastwood, dans La Dernière Cible
L'année suivante, en 1987, Eastwood prend la décision de se séparer de son ami d'enfance et producteur de la Malpaso depuis treize ans, Fritz Mane. Les deux hommes ont eu plusieurs différends depuis quelque temps, notamment sur le dernier tournage. Eastwood en profite pour s'éloigner un peu du cinéma et se concentrer sur sa fonction de maire. Il revient en 1988 à la Malpaso avec un nouveau film qu'il juge risqué cependant : Bird. Passionné par le jazz depuis sa jeunesse, il décide, avec Bird, de consacrer son premier film biographique au saxophoniste Charlie Parker. Pour la deuxième fois de sa carrière, Eastwood reste derrière la caméra. Le réalisateur fait un gros effort pour essayer de recréer l'ambiance propre à Parker, autant dans la musique que dans les décors. Le film est projeté au e Festival de Cannes. Il y est assez bien reçu, et on peut lire des critiques comme « une œuvre spectaculaire… exceptionnelle — une épopée intime ». Le film reçoit même une récompense, le Prix d'interprétation masculine. Bien qu'Eastwood fasse partie des favoris pour le prix de la mise en scène, le jury l'attribue finalement à Fernando Solanas, pour son film Le Sud. Cette année-là, le Times publie, faussement compatissant, au sujet d'Eastwood : « Eastwood a monté consciencieusement les marches du Palais ; il était toujours aussi grand, mais sa dignité courbait le dos. La barre des attentes avait été fixée trop haut, mais il aurait dû obtenir une récompense pour le simple fait d'avoir fait briller son charme magnétique sur Cannes. » En parallèle, Eastwood produit un documentaire sur le jazzman Thelonious Monk : Thelonious Monk: Straight, No Chaser.
En 1988, il annonce qu'il ne se représentera pas aux élections de Carmel, préférant passer du temps avec ses enfants déjà adolescents,. En effet, son mandat de maire est mitigé. Beaucoup ont apprécié ses actions, tels que le Carmel Pine Cone ou encore la législature de l'État de Californie, mais certains habitants de la ville parlent de mettre en place un système empêchant une star de se présenter au poste de maire. Eastwood se rend alors à San Francisco pour tourner le dernier volet de L'inspecteur Harry : il doit résoudre une série d'assassinats macabres de célébrités. Le film s'intitule L'inspecteur Harry est la dernière cible et l'on y remarque la présence de Jim Carrey dans l'un de ses premiers rôles. Toutefois, ce dernier épisode de la saga n'est pas une réussite commerciale, les recettes finales atteignant à peine la moitié de celles réalisées avec Le Retour de l'inspecteur Harry en 1983.
Le déclin ?

Statue de John Huston qu'interprète Eastwood dans Chasseur blanc, cœur noir
La Warner s'inquiète au sujet d'Eastwood : La Dernière Cible et Bird n'ont pas eu les résultats souhaités. Aussi murmure-t-on qu'Eastwood ne rapporte plus ce qu'il rapportait au box-office, malgré le coût élevé des tournages en extérieur de ses films. Durant cette période, Eastwood se sépare de Sondra Locke, leur relation se dégradant depuis que Locke a réalisé son film Ratboy en 1986. Eastwood refuse de donner quoi que ce soit à son ex-compagne, puisqu'elle est toujours mariée à un autre homme. Elle décide donc d'intenter une action en justice pour obtenir un dédommagement de 70 000 000 $, mais ils arrivent à trouver un arrangement négocié. Pink Cadillac met en scène un chasseur de primes incarné par Eastwood, confronté à un enfant enlevé et une bande de suprémacistes blancs. Le tournage se déroule durant l'automne 1988, et le film sort le 26 mai 1989 dans plus de 2 000 salles. Variety juge le film « médiocre » et le qualifie de « film rasoir de 122 minutes ». Devant ces critiques, le film quitte rapidement l'affiche et se retrouve parmi les pires de la décennie pour Malpaso.
L'année suivante, en 1990, Eastwood entame un nouveau tournage, celui de Chasseur blanc, cœur noir. Cela faisait un moment que le film était en pré-production. Eastwood part en Afrique, au Zimbabwe, pour faire les repérages et démarre les prises de vues qui durent deux mois. Cette fois, il interprète le réalisateur américain John Huston. En rentrant aux États-Unis, Eastwood commence une relation sérieuse avec France Fisher,,. Et pour la troisième fois en sept ans, il se rend au Festival de Cannes. L'accueil est moins enthousiaste que pour Bird, selon Jack Matthiews du Los Angeles Times. Le film est également projeté au Festival du film de Telluride du Colorado. Le froid jeté par la critique amène Eastwood à enchaîner rapidement avec La Relève, un nouveau film populaire,. Eastwood est encore une fois acteur principal et réalisateur dans ce film d'action. Si certains critiques comme Gary Giddins du Village Voice trouvent le film « tout simplement génial », d'autres sont écœurés de voir Eastwood produire quelque chose d'aussi « étonnamment vide de sens ». Du côté du public, le film n'est pas accueilli avec beaucoup d'enthousiasme non plus et rapporte à peine un peu plus de 20 000 000 $.
Un prix tant attendu

Lac Moraine (Alberta) où a lieu le tournage d’Impitoyable
En août 1991, le Los Angeles Times annonce le prochain film de Clint Eastwood, un western dont le titre serait Impitoyable. L'histoire est écrite par David Webb Peoples, qui a déjà travaillé sur un documentaire nommé aux Oscars et sur Blade Runner, réalisé par Ridley Scott en 1982. Cela faisait longtemps que le scénario de Peoples avait été réservé par Warner, aussi ce dernier désespérait-il de le voir un jour adapté au cinéma. Ce n'est que six ans après que tout a été officialisé. Eastwood déclare durant la pré-production du film : « Je le savourais, parce que je me disais que ce serait sans doute le dernier du genre, le dernier film de ce type que je ferais. » Le film est vu par la presse comme « un western révisionniste, un film violent pour démythifier le meurtre ». Ce trait est d'ailleurs exagéré pour la publicité du film.
À la différence de la plupart de ses autres films, Clint offre ici à d'autres acteurs des rôles tout aussi importants que le sien : notamment à Richard Harris, qui incarne English Bob, le héros de dime novels, Morgan Freeman et Saul Rubinek. Le personnage interprété par ce dernier rappelle le comportement prudent qu'a adopté Eastwood envers la critique depuis un certain temps. Gene Hackman est aussi pressenti, mais il refuse tout d'abord : il ne voit pas ce qu'Eastwood admire dans ce scénario qu'il trouve trop violent. Mais Eastwood se montre assez persuasif pour que l'acteur qui a déjà remporté un Oscar accepte finalement le rôle. Frances Fisher fait également partie de la distribution sûrement grâce à sa relation avec Eastwood. Le film est tourné en Alberta, au Canada, en août 1991. L'équipe technique comporte encore Lennie Niehaus, Joel Cox et Jack Green, des habitués de l'équipe Eastwood. Ce film est différent des précédents : il semble qu'Eastwood soit plus attentif au jeu des acteurs en insistant sur les répétitions et le nombre de prises effectuées. En janvier 1992, le film est présenté au ShoWest, puis une avant-première a lieu à New York, à laquelle cent cinquante médias sont invités. Grâce à un bon marketing de Warner, le film est réservé dans plus de 2 000 salles en Amérique. Durant sa première semaine d'exploitation, il surpasse tous les espoirs, réalisant 14 000 000 $ de recette. Variety l'appelle « un western classique qui marquera les mémoires » et le Los Angeles Times « le meilleur western depuis 1956 ». Tous les médias s'accordent sur l'apogée d'Eastwood.

Wolfgang Petersen, qu'il rencontre durant la cérémonie des Oscars, et pour qui il jouera dans In the Line of Fire
Ce film permet au réalisateur de remporter le prix annuel du meilleur réalisateur délivré par la National Society of Film Critics. Il est aussi nommé meilleur film de l'année par la Boston Society of Film Critics. Il figure dans plus de deux cents listes des dix meilleurs films de l'année. Eastwood remporte le prix de meilleur réalisateur à la Directors Guild of America. Le mois suivant, il est nommé à neuf Oscars dont le prix du meilleur réalisateur et du meilleur acteur. L'Oscar du meilleur acteur est finalement remporté par Al Pacino pour Le Temps d'un week-end, mais Eastwood repart avec le prix du meilleur réalisateur, un prix tant attendu dans sa carrière. Durant son discours, il remercie tous les artistes et techniciens qui ont participé au film, mais également les critiques. Plus tard dans la soirée, Eastwood remonte sur scène pour recevoir l'Oscar du meilleur film pour lequel il remercie Warner. Il salue sa mère présente durant la cérémonie,, et Arthur Lubin, qu'il remercie pour avoir lancé sa carrière.
Le film suivant d'Eastwood, Dans la ligne de mire, raconte l'histoire d'agents secrets formés pour protéger le Président. C'est Castle Rock Entertainment qui possède les droits du film. La société projette d'engager Wolfgang Petersen pour la réalisation et Eastwood dans le rôle principal, celui d'un agent qui s'effondre moralement quand il réalise qu'il a réussi à sauver la première dame mais pas le président, John Fitzgerald Kennedy. Tous deux acceptent. Ils se sont rencontrés à l'occasion de la sortie d'Impitoyable ; ils ont alors parlé durant deux heures, du prochain film qu'ils allaient faire, de leur approche en tant que cinéastes, et du cinéma en général. C'est la première fois qu'Eastwood ne tourne pas pour Warner. La distribution comprend également John Malkovich et Rene Russo. Petersen avoue au sujet d'Eastwood : « Il est meilleur acteur que je ne le pensais avant de commencer le film. Et plus on tournait, plus je prenais plaisir à m'imaginer toutes les facettes qu'il pouvait encore révéler. » Le film devient le plus rentable de toute la carrière d'Eastwood, rapportant au total plus de 200 000 000 $.
Clint Eastwood, le romantique

Eastwood au festival de Cannes 1994.
Face au réel succès qu'il rencontre, Eastwood cherche tout de suite un nouveau projet. Ce sera Un monde parfait, écrit par John Lee Hancock. Le script est réservé par Mark Johnson pour être montré à Steven Spielberg. Ce dernier ne peut toutefois pas le tourner à la suite d'autres engagements. Alors, le script parvient à Warner où il est montré à Eastwood. L'histoire lui rappelle Seuls sont les indomptés, un western contemporain sorti en 1962. Un monde parfait met en scène un fugitif qui s'évade et prend en otage un jeune enfant. Spielberg pensait que Clint Eastwood était l'acteur idéal pour interpréter ce fugitif, mais lui-même se trouve trop âgé ; il accepte cependant de réaliser le film. Johnson a l'idée d'engager Kevin Costner pour le rôle principal. Eastwood adhère à ce choix : « Je ne crois pas que ce soit sa tasse de thé. Mais c'est parfois dans ces moments-là que les acteurs font leurs meilleures prestations. » Costner accepte le rôle à condition de jouer face à Eastwood. Pour améliorer les chances de réussite du film, il pense qu'Eastwood pourrait jouer le représentant de la loi, mais le rôle n'est pas suffisamment étoffé. Costner essaye donc de développer le rôle avec Hancock, et Eastwood accepte finalement à son tour de jouer dans le film. Le tournage est agrémenté de quelques tensions, car Eastwood réalise généralement ses films rapidement, avec peu de prises, à la différence de Costner qui est perfectionniste. Eastwood dit à son sujet : « il n'arrête pas de pinailler ». Le tournage prend donc du retard. Costner le quitte même, mais Eastwood décide de lui montrer qui gouverne en tournant avec sa doublure. Le filmage s'achève tandis que naît le sixième enfant d'Eastwood : Francesca Ruth Fisher. Durant cinq semaines, Clint se retire dans son ranch pour se consacrer à sa femme et son nouveau-né. Joel Cox profite de cette période pour monter Un monde parfait. Le film rapporte finalement 150 000 000 $ en Amérique.
La même année, Eastwood est élu membre du British Film Institute de Londres et le Museum of Modern Art de New York intègre dans ses archives la collection Clint Eastwood. C'est également en 1993 qu'Eastwood bat des records au box-office avec ses trois films, Impitoyable, Dans la ligne de mire et Un monde parfait, et que les critiques sont plus que jamais unanimes à son sujet. Eastwood rajoute qu'il « [a] mûri » durant cette période. En 1994, il préside le jury du Festival de Cannes qui récompense Pulp Fiction du réalisateur indépendant Quentin Tarantino. Il reçoit aussi la médaille de Commandeur de l'ordre des Arts et des Lettres.
« Les rôles qu'Eastwood a interprétés, et les films qu'il a mis en scène, ont largement influencé la culture des États-Unis du dernier quart du siècle, sa fantaisie et ses réalités. »
— Edward Gallafent commentant l’impact d'Eastwood sur les films des années 1970 aux années 1990
Durant le Festival, il annonce qu'il a accepté de jouer dans l'adaptation du best-seller The Bridges of Madison County (Sur la route de Madison) sous la direction de Bruce Beresford. Le roman met en scène une femme d'âge mûr, épouse d'un fermier parti pour la foire de l'État. Un photographe en visite dans la région s'éprend d'elle et ils vivent un grand amour romantique sans lendemain. Spielberg est à nouveau impliqué dans le projet puisque sa société Amblin Entertainment possède les droits d'adaptation du livre. La pré-production est ponctuée de conflits entre Beresford et Eastwood, notamment pour le choix de l'actrice principale. Beresford quitte donc le projet, et c'est Eastwood qui le remplace. Il visite rapidement les lieux repérés par Beresford et choisit comme partenaire Meryl Streep, qui, initialement peu intéressée par le projet de Beresford, accepte finalement le rôle proposé.

Le Roseman Bridge du Comté de Madison que doit photographier le personnage d'Eastwood dans le film
Meryl Streep est un choix crucial pour son pouvoir d'attraction sur le public féminin. Avant le tournage, Eastwood ne lui dit rien d'autre que « sois naturelle », et ce, malgré le fait que le personnage ait grandi en Italie. Elle est étonnée qu'Eastwood souhaite tourner ce film en plus d'y jouer le premier rôle. Elle rajoute d'ailleurs, au cours d'une interview, qu'elle est extrêmement surprise par la confiance qu'il a en lui pour ce film. Le tournage commence à la mi-septembre à Des Moines. Kyle Eastwood fait un caméo dans le film : il interprète le rôle d'un bassiste dans un café où Robert Kincaid et Francesca se trouvent. Ignorant la demande du réalisateur, Meryl Streep débute le tournage en prenant un accent italien pour lequel elle s'était entraînée plusieurs semaines. Durant la première moitié des prises de vues, Eastwood ne dit rien à l'actrice, qui se demande si elle joue réellement ce qu'il attend d'elle. Aussi, ce dernier lui dit-il un jour : « Tu sais, je ne dis jamais rien, sauf quand je n'aime pas. » Le tournage dure six semaines, au lieu des dix prévues initialement. Au cours des interviews pour la promotion du film, le réalisateur déclare qu'il a essayé de « garder beaucoup de spontanéité et de sincérité ». On peut lire dans les journaux qu'une « intense amitié est née entre les deux stars ».
C'est durant le tournage de Sur la route de Madison qu'Eastwood se sépare de Frances Fisher. Elle désirait plus que tout obtenir un rôle qui lui permettrait d'être auprès d'Eastwood, mais également de faire connaissance avec l'une de ses actrices préférées. Il a refusé, pour ne pas faire les mêmes erreurs qu'avec Locke. Fisher découvre deux semaines plus tard Eastwood et Dina Ruiz à la une d'un magazine. Sur la route de Madison sort en juin 1995. Warner profite de ce film pour proposer la candidature d'Eastwood au Irving G. Thalberg Memorial Award. Le soir de la cérémonie des Oscars, Eastwood gagne finalement le prix, le discours du président de la le décrivant comme « un cinéaste très actif ». Malgré la récompense, les exploitants sont sceptiques quant au film ; seules 1 805 salles le projettent. Toutefois le succès populaire est au rendez-vous et, à la fin de l'été, le film est toujours projeté dans un millier de salles. En fin d'exploitation, Sur la route de Madison rapporte plus de 70 000 000 $ rien qu'aux États-Unis. Outre le succès public, la critique est agréablement étonnée par le film : le New York Times évoque « une émouvante et élégiaque histoire d'amour au cœur de l'œuvre dégoulinante » et le New York Daily News « des passages […] plus puissants que tout ce que l'on avait eu l'occasion de voir au cinéma auparavant ». On découvre qu'Eastwood est un romantique, jusque là plutôt caché derrière l'homme d'action. Il participe aussi à la bande originale du film dont il compose le thème principal. L'album se situe en tête de ventes de disques jazz, et permet la formation de Malpaso Records.
De mauvais accueils
En septembre de la même année, le San Francisco Chronicles surprend tout le monde en dévoilant que Clint Eastwood vient de demander Dina Ruiz en mariage. Cette période est marquée par le profond intérêt des tabloïds pour l'acteur, qui n'hésite pas à poursuivre en justice nombre de ceux qui le dépeignent comme un coureur de jupons. Aussi, le mariage est-il célébré dans la plus grande intimité à Las Vegas. Le journal People l'annonce avec ce titre ironique : « Se faire plaisir : Clint Eastwood épouse une jeune présentatrice de 30 ans, et pas sous la menace d'une arme ». Comme People est détenu par Time Warner, la société mère de Warner Brothers, Eastwood est furieux. Il menace même d'abandonner son prochain projet, Les Pleins Pouvoirs. Il s'agit d'un thriller politique adapté du best-seller éponyme sorti en 1995. Le magazine publie peu après des excuses publiques, et Eastwood ne met pas sa menace à exécution.

Cimetière de Savannah où a lieu le tournage de Minuit dans le jardin du bien et du mal
Le film met en scène un cambrioleur vieillissant qui surprend, au cours d'un cambriolage, des ébats sadomasochistes finissant par un meurtre dont l'auteur n'est autre que le Président. On a proposé le script à Eastwood, qui souhaite jouer le rôle du cambrioleur. Toutefois, il demande des modifications : il ne doit pas mourir dans le film. Goldman, qui a tout fait pour travailler avec lui, obéit finalement par dépit. Il travaille un mois à la correction du script. La distribution comporte de nombreuses célébrités dont Gene Hackman, Judy Davis et Ed Harris. Alison Eastwood, la fille du réalisateur, fait une courte apparition dans la première scène du film (elle observe le cambrioleur), et Kimber Eastwood apparaît en guide touristique. Le tournage se termine dix-sept jours avant la date prévue. Le 12 décembre naît Morgan Colette Eastwood, fille d'Eastwood et de Ruiz, au terme de quarante-cinq heures de travail. Un mois plus tard, Les Pleins Pouvoirs sort dans 2 568 cinémas aux États-Unis. Le succès est mitigé en Amérique, le film ne rapportant que 50 000 000 $, alors qu'il fonctionne très bien à l'étranger. La critique le trouve plutôt décevant, voire bâclé, comparé à Impitoyable, par exemple. Il devait faire la clôture du Festival de Cannes, mais Eastwood s'est désisté, de crainte d'un accueil négatif ; pourtant, lors de sa distribution en France, le film est clairement acclamé.
Minuit dans le jardin du bien et du mal, le projet suivant d'Eastwood, est également tiré d'un roman. Hancock, scénariste d'Un monde parfait, est embauché pour combler les manques du script : il doit créer une ligne narrative et mettre en valeur des personnages principaux. La Warner, insatisfaite du projet, le met en attente, mais Hancock l'envoie à Eastwood, qui le rappelle immédiatement pour lui dire qu'il souhaite le réaliser. La mise en production est annoncée en février 1996. La distribution comprend John Cusack, Kevin Spacey, Lady Chablis, qui joue son propre rôle, Alison Eastwood, qui, déterminée à poursuivre dans le cinéma, obtient ici un rôle plus important, mais Clint Eastwood n'y interprète aucun rôle. Le tournage a lieu à Savannah et se termine en six semaines. Lors de la promotion du film, personne n'évoque le fait qu'Eastwood, à plusieurs reprises dans le passé, a dénigré les homosexuels, dont le mari de Locke, alors que l'intrigue de Minuit dans le jardin du bien et du mal tourne autour du meurtre d'un gay. Lors de sa sortie aux États-Unis, le film, qui dure 155 minutes, est un échec : il rapporte à peine 25 000 000 dollars et les critiques n'apprécient pas du tout le jeu de Cusack. En France en revanche, en 1998, Eastwood reçoit un César d'honneur pour toute sa carrière.

Eastwood à la NASA, qui lui accorde le tournage de Space Cowboys
Si Eastwood se dit à la recherche d'un nouvel Inspecteur Harry, il s'engage toutefois sur la production de Créance de sang, tiré du roman de Michael Connelly, dont le script a enthousiasmé Malpaso et Warner. Mais, le roman venant à peine de sortir, Créance de sang est repoussé de quelques mois. Clint Eastwood se lance alors dans la réalisation de Jugé coupable, dont les droits sont détenus par Richard D. Zanuck, qui prend en charge la production avec son épouse, Lili Zanuck. Créance de sang est distribué au printemps 1999. Ce film se rapproche par son action de L'Inspecteur Harry. Il met en scène un tueur en série qui prend pour cible des donneurs d'organes, pour faciliter l'opération de son ennemi juré, un profiler du FBI, et lui permettre ainsi de reprendre son travail. L'assassin, alors, n'aurait plus qu'à le provoquer à nouveau pour lui échapper ensuite. La distribution comprend Eastwood lui-même, Brian Helgeland, Anjelica Huston, Jeff Daniels et Dina Ruiz. Le film déçoit autant la critique que le public : il rapporte à peine 17 000 000 $ malgré ses 1 852 copies. On peut lire dans le magazine Rolling Stone que les réalisations d'Eastwood depuis 1992 sont « moyennes… voire médiocres… voire pires ». On va jusqu'à suggérer qu'Eastwood est en fin de carrière. Au cours d'interviews données à l'occasion de la sortie du film, Eastwood déclare qu'il « est trop vieux pour ces gamineries » au sujet d'un hypothétique futur Harry.
Au début des années 2000, alors qu'il a déjà 70 ans, Eastwood entame le tournage de Space Cowboys en accord avec la NASA. Le film met en scène quatre vieux pilotes d'essai qui doivent se rendre dans l'espace pour s'occuper d'un satellite devenu incontrôlable. La distribution est composée de Tommy Lee Jones, Donald Sutherland, James Garner et Eastwood lui-même. Warner entre en contact avec Industrial Light & Magic pour les effets spéciaux. Les critiques considèrent le film comme « un divertissement très agréable sans prétention ». Il rapporte finalement 100 000 000 $ en Amérique. En septembre de la même année, à la Mostra de Venise, Eastwood se voit récompensé d'un Lion d'or d'honneur pour toute sa carrière.
Une carrière réfléchie

Dennis Lehane en 2006
Depuis ses débuts dans Rawhide, Clint Eastwood a fait beaucoup de chemin. Il ne marmonne plus à l'écran, réalise ses propres films et engage de véritables acteurs de cinéma, plutôt que des acteurs de télévision méconnus. S'il semble à l'apogée de sa carrière malgré l'échec relatif de ses trois derniers films, l'acteur, réalisateur et producteur ne souhaite pas s'arrêter là. Il a en vue le roman de Dennis Lehane : Mystic River, publié en 2001. Il met en scène trois anciens amis liés par un sombre incident. Le meurtre de la fille de l'un d'eux va les réunir à nouveau. Eastwood téléphone personnellement à Lehane pour réserver les droits d'adaptation. Pour la quatrième fois de sa carrière, il s'occupe seulement de la mise en scène du film. Le tournage se déroule à Boston avec Sean Penn, Kevin Bacon et Tim Robbins dans les rôles principaux. Les trois acteurs se réunissent chaque soir après le tournage pour répéter les scènes du lendemain, avec la bénédiction d'Eastwood, qui ne donnait que rarement des instructions pour l'interprétation des personnages. Le tournage dure plus longtemps que pour ses précédentes productions. Et pour la première fois il s'occupe personnellement de la musique du film. Mystic River est projeté en avant-première au Festival de Cannes de mai 2003, et, pour beaucoup de critiques, sauve « le pire festival qu'il y ait jamais eu ». Le film est projeté en Amérique pour la première fois durant le New York Film Festival. Puis, en septembre, officiellement distribué partout dans le pays. Au début, il engendre des recettes juste convenables, sans doute à cause de l'aspect sombre de l'histoire. Mais le succès va crescendo au fur et à mesure que le film est projeté dans de nouvelles salles, et finalement les recettes mondiales atteignent 150 000 000 $. Mystic River est nommé à six reprises aux Oscars, mais n'en remporte que deux, concurrencé par le dernier volet de la trilogie du Seigneur des anneaux : Le Retour du roi.
« L'ingrédient principal du film – Mystic River — est Clint Eastwood. C'est un vrai artiste dans tous les sens du terme. Malgré toutes ces années dans le milieu cinématographique et ses films légendaires, il a toujours tout fait pour nous rendre le travail agréable et nous mettre à l’aise lors de la prise, nous traitant comme ses pairs. […] C'était une expérience immense. »
— Tim Robbins à propos du tournage de Mystic River
Insérer un paragraphe

Eastwood à une conférence de presse en Allemagne pour Lettres d'Iwo Jima
La production qui suit, Million Dollar Baby, est l'adaptation d'un roman écrit par Haggis. L'histoire relate la vie d'un vieil entraîneur de boxe et d'une jeune boxeuse novice. Haggis écrit une première version adoptée par la Warner, qui pense lui en proposer la mise en scène et donner le premier rôle à Eastwood. Mais Eastwood décide de réaliser le film en plus d'y interpréter le rôle du vieil entraîneur. Hilary Swank est embauchée pour jouer le rôle de la jeune boxeuse et Morgan Freeman pour interpréter le gardien du gymnase, vieil ami de l'entraîneur. Le tournage a lieu à Los Angeles au début de l'année 2004. Eastwood endosse à nouveau le rôle de compositeur, en plus de ceux d'acteur, réalisateur et producteur. À sa sortie en décembre le film soulève une controverse parce qu'il se conclut sur l'euthanasie de Maggie Fitzgerald, le personnage joué par Hilary Swank. Eastwood réplique alors qu'il « n'est pas nécessaire d'être pour l'inceste pour aller voir Hamlet». Les aspects sentimentaux et populistes de Million Dollar Baby ne le font pas décoller au box-office immédiatement. L'intrigue effraye Warner, qui s'associe même à Lakeshore Entertainment pour distribuer le film. Million Dollar Baby ne fait donc pas l'unanimité et n’est projeté que dans 147 salles avant que toutes les nominations aux cérémonies de récompenses et aux festivals ne soient annoncées, ne rapportant que 8 300 000 $. En revanche, dès que les nominations sont dévoilées, il commence à soulever l’engouement, rapportant 56 600 000 $ juste avant les Oscars, et 35 600 000 $ après. Il permet à Eastwood d'être nommé à l'Outstanding Directorial Achievement de la , et de remporter quatre Oscars sur sept nominations : celui du meilleur film (pour la deuxième fois dans sa carrière), du meilleur réalisateur (pour la deuxième fois également), de la meilleure actrice pour Hilary Swank et du meilleur second rôle pour Morgan Freeman.
Eastwood et Spielberg se voient lors du repas qui suit la cérémonie des Oscars, et parlent de Mémoires de nos pères, une adaptation du livre de James Bradley et Ron Powers sur le petit groupe de soldats qui a planté le drapeau américain à Iwo Jima. Spielberg offre à Eastwood les droits d'adaptation et lui propose de coproduire le film avec ses sociétés DreamWorks et Amblin Entertainment. Paul Haggis s'occupe d'écrire le script et donne en octobre 2004 une première version, qui est adoptée. Au cours de ses recherches pour le film, Eastwood tombe sur un recueil de lettres de guerre japonaises. Il demande alors à faire un second film qui décrirait les mêmes évènements que Mémoires de nos pères, mais du point de vue japonais : Lettres d'Iwo Jima. Il s'agit du premier film américain à montrer la guerre du point de vue ennemi. Les frais de tournage, salaires, effets spéciaux, matériel militaire, font de Mémoires de nos pères le film le plus coûteux de la carrière d'Eastwood, avec un budget de 90 000 000 $. Le deuxième, Lettres d'Iwo Jima, est également complexe du fait qu'il est tourné en japonais avec des acteurs japonais, mais le budget de celui-ci n'est estimé qu'à 20 000 000 $ et le tournage dure à peine plus d'un mois. Henry Bumstead, le directeur artistique d'Eastwood depuis Impitoyable meurt pendant le tournage, et il est remplacé par James J. Murakami. Mémoires de nos pères n'enthousiasme ni le public, ni les critiques, qui trouvent qu'il ne se démarque pas des autres films sur la Seconde Guerre mondiale, et les recettes n'atteignent pas le budget investi. Le deuxième, dont le style est plus novateur (les vues aux couleurs ternes se rapprochent du noir et blanc et les scènes de combats sont plus émouvantes qu'effrayantes) marque une consécration d'Eastwood. L'œuvre apparaît sur toutes les listes des meilleurs films de l’année. Le film est nommé aux Oscars mais n’en remporte qu'un, laissant celui du meilleur film, du meilleur réalisateur et du meilleur scénario aux Infiltrés de Martin Scorsese.

Eastwood et sa femme, Dina Ruiz, aux Oscars
Clint Eastwood : le réalisateur

Angelina Jolie et Eastwood lors du Festival de Cannes 2008.
Warner, qui s'est méfié de Mystic River et de Million Dollar Baby avant de les produire, comprend peu à peu qu'Eastwood sait ce qu'il fait, et surtout qu'il est une valeur sûre. En 2007, Eastwood accepte de produire et réaliser L'Échange. C'est le septième projet auquel il participe sans interpréter aucun rôle. Le film raconte l'histoire d'une mère, interprétée par Angelina Jolie, dont le fils a été enlevé. Elle fait tout pour le retrouver, en dépit de la corruption des forces de l’ordre. Le scénario, inspiré de faits réels survenus en 1928, est de Joseph Michael Straczynski, et John Malkovich fait partie de la distribution. Le tournage s'effectue assez rapidement, de même que la postproduction, pour que le film soit prêt à temps pour le Festival de Cannes 2008. En France, le film est chaleureusement accueilli, à la différence des États-Unis où New York Times le juge « maladroit et hautement ambigu ». Si le film repart de Cannes sans aucun prix, Clint Eastwood et Catherine Deneuve reçoivent le Prix du e Festival de Cannes pour l'ensemble de leur carrière, bien que lui-même ne soit plus à Cannes lors de la cérémonie de remise des prix.
Durant le festival, Spike Lee, également présent pour la promotion de son film Miracle à Santa Anna (Miracle at St. Anna), entame une controverse, reprochant à Eastwood l'absence de soldats Afro-Américains dans ses deux films Mémoires de nos pères et Lettres d'Iwo Jima. Eastwood lui répond violemment lors d'une interview dans The Guardian : « il m’a critiqué quand j’ai fait Bird. Pourquoi est-ce qu'un type blanc fait ce genre de film ? Parce qu'il fallait bien que quelqu'un le fasse […] Ce ne sont pas [les Afro-américains] qui ont érigé le drapeau américain ». Les journalistes soutiennent Eastwood, et Lee s'incline finalement, le mois suivant. L'Échange est distribué pour la première fois en Amérique lors du New York Film Festival et vingt jours après dans tout le pays. En fin de distribution, le film engendre une recette totale de 113 000 000 $, mais il marche mieux à l’étranger (77 280 454 $) qu'aux États-Unis (35 739 802 $). Il demeure toutefois six semaines dans le top 10 du box-office national. De son côté, la critique est assez mitigée. Le jeu d'Angelina Jolie semble avoir enthousiasmé les médias, comme celui de beaucoup de seconds rôles. Cependant, le scénario « sonne faux » et la mise en scène est « lourde » pour The Wall Street Journal. Le film est nommé à plusieurs reprises aux BAFTA Awards, au Saturn Award ainsi qu'aux Critics Choice Awards, mais ne remporte finalement aucun prix.
Alors que L'Échange est en postproduction, Bill Berger reçoit un script de Nick Schenk. À la fin des années 1990, ce dernier se familiarise avec l'histoire et la culture des Hmong alors qu'il travaille dans une usine du Minnesota. Il apprend comment, ayant soutenu les forces vietnamiennes et leurs alliés américains durant la Guerre du Viêt Nam, ils ont terminé dans des camps de réfugiés, à la merci des forces communistes du Nord, quand les troupes américaines se sont retirées. Des années plus tard, il décide d'écrire une histoire impliquant un vétéran de la guerre de Corée, nommé Walt Kowalski, dont les nouveaux voisins sont une famille hmong. Plusieurs sociétés de production préviennent Schenk qu'il ne pourra pas produire son film. Il ne les écoute pas et envoie son script à Berger, alors producteur de Warner. Berger présente ainsi l'histoire à Eastwood, pour diriger et jouer dans ce film, ce qu'il accepte directement, trouvant « dans ce rôle marrant un réel challenge ». Le tournage de Gran Torino commence en juillet 2008 à Highland Park et Détroit puis à Warren, Royal Oak et Grosse Pointe Park. Clint Eastwood désire que la distribution fasse appel à de vrais Hmong, organise-t-on plusieurs auditions dans des communautés hmong. Finalement, Bee Vang est engagé à Saint Paul et Ahney Her est engagée à Détroit ; ils interprètent les deux principaux rôles hmong. Scott Eastwood apparaît également dans le film. Kyle Eastwood compose la musique du film. Le tournage est rapidement terminé, et Gran Torino est distribué le 9 décembre 2008 en Amérique, et le 25 février en France. Durant sa première semaine aux États-Unis, le film bénéficie d'une sortie limitée dans 6 salles seulement, engrangeant une recette de 391 639 $ ; puis il sort dans 2 808 salles et réalise une recette de 39 957 281 $, se plaçant premier du box-office. Après 27 semaines, le film rapporte finalement 148 095 302 $ aux États-Unis. En fin d'exploitation, les recettes mondiales atteignent 269 958 228 $, faisant de Gran Torino le plus grand succès d'Eastwood en tant que réalisateur. Le cinéaste remporte le Blue Ribbon Awards du meilleur film étranger ainsi que le César du meilleur film étranger. Le National Board of Review déclare Eastwood meilleur acteur de l’année. C'est également Gran Torino qui marque la consécration d'Eastwood à Cannes, puisque les dirigeants du Festival lui remettent la Palme d'honneur pour sa carrière lors de la promotion du film à Paris. Clint Eastwood déclare que Kowalsky sera son dernier rôle à l’écran, une promesse qu'il rompra avec Une Nouvelle Chance.

Robben Island, prison où Nelson Mandela a été enfermé, et lieu de tournage d'Invictus
En 2009, Eastwood tourne Invictus, avec Morgan Freeman dans le rôle de Nelson Mandela et Matt Damon dans le rôle du capitaine de l'équipe de rugby à XV sud africaine, François Pienaar. L'histoire est tirée d'un livre de John Carlin, Playing the Enemy : Nelson Mandela and the Game that Made a Nation. Ce dernier vend les droits d'adaptation à Morgan Freeman l'année de la sortie du livre, en 2008. Elle retrace la libération de Mandela ainsi que son arrivée à la Présidence, puis sa décision d'unir son peuple, qu'il soit noir ou blanc, à travers la Coupe du monde de rugby à XV 1995 et l’équipe des Springboks. Anthony Peckham et Eastwood se sont rendus à Barcelone pour rencontrer John Carlin et discuter de l’adaptation du livre. Le rôle de Mandela a d'emblée été offert à Freeman. Bien qu'il soit plus petit que Pienaar, son rôle est proposé à Matt Damon. Ce dernier suit un entraînement intensif avec Chester Williams, un joueur de l’équipe des Springboks de 1995. Le tournage commence en mars 2009 au Cap et se termine en mai. Le film est distribué le 11 décembre 2009 aux États-Unis et le 13 janvier 2010 en France. Il est accueilli chaleureusement tant par le public que par la critique. Les recettes mondiales avoisinent les 123 000 000 $ et la critique acclame la fraîcheur du film ; on peut lire dans le Chicago Sun-Times « c'est un très bon film, il a de grands moments d'émotion ». En parallèle, Eastwood est élu personnalité du cinéma préférée des Américains selon le sondage Harris Interactive.
Depuis 2010 : entre acclamations et échecs
Bien qu'Eastwood approche de ses quatre-vingts ans, il enchaîne directement avec un nouveau projet (et son premier film fantastique), Au-delà (Hereafter). Il s'agit d'un thriller prenant la forme d'un film choral, écrit par Peter Morgan. Le tournage débute le 19 octobre 2009 en France, avec Matt Damon, Cécile de France, Lyndsey Marshal et Bryce Dallas Howard. Variety le décrit comme un thriller « dans la veine de Sixième Sens » en référence au film réalisé par M. Night Shyamalan en 1999. Toutefois le film n'aborde pas ce thème commun de la même manière. Au-delà, qui raconte l'histoire de trois personnes qui sont touchées par la mort de différentes manières. Cécile de France, qui incarne une journaliste française, prend conscience de l'au-delà alors qu'elle est victime d'un tsunami ; Frankie McLaren prend conscience de la mort quand son frère se fait renverser par une voiture et enfin, Bryce Dallas Howard, dont les parents sont morts. Matt Damon interprète un médium capable d'entrer dans l'au-delà, de communiquer avec les morts. Ce dernier a décidé de mettre de côté ce don qu'il considère comme une « malédiction ». Aussi, Au-delà aborde les questions que soulève la mort avec subtilité. Le film ne prend pas parti sur son existence `
